/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: George F. Riley<riley@ece.gatech.edu>
 */

// Program to read and parse an XML file describing a network animation,
// then animate it.
// George F. Riley, Georgia Tech, Spring 2010

#include <iostream>
#include <fstream>
#include <sstream>
#include <string>
#include <map>

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <pthread.h>

#include <xercesc/util/XMLString.hpp>
#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/util/BinFileInputStream.hpp>
#include <xercesc/framework/StdInInputSource.hpp>
#include <xercesc/framework/LocalFileInputSource.hpp>
#include <xercesc/parsers/SAXParser.hpp>
#include <xercesc/sax/HandlerBase.hpp>
#include <xercesc/sax/AttributeList.hpp>
#include <xercesc/sax/SAXParseException.hpp>
#include <xercesc/sax/SAXException.hpp>

#include "Node.h"
#include "WiredLink.h"
#include "Packet.h"
#include "ThreadInfo.h"
#include "MainInfo/MainInfo.h"

#include <QApplication>
#include <QWidget>
#include <QTimer>
#include <QPainter>
#include <QCoreApplication>
#include <QDesktopWidget>
#include <QHBoxLayout>
#include "NetModel.h"
#include "NetView.h"

#include "XML/animparser.h"
#include "XML/animdochandlers.h"
#include "XML/PartialStream.h"

#include "sys/time.h"

#ifdef WITH_NS3
#include "Simulation/private/simulation.h"
#endif

using namespace std;
using namespace xercesc;
using namespace netanim;
// The following are for thread coordination, insuring all reader
// threads advance in synchronization.

netanim::MainInfo* mainInfo;  // Shared infomation

void WaitForAdvance(unsigned myIndex)
{ 
    // Wait for the main to signal us to continue
    ThreadInfo& myInfo = mainInfo->threadInfo[myIndex];
    pthread_cond_wait(&myInfo.cond, &myInfo.lock);

}

void SignalAdvanced(unsigned int myIndex)
{
    ThreadInfo& myInfo = mainInfo->threadInfo[myIndex];
    myInfo.advancedTo = myInfo.lastXMLTime;
    pthread_mutex_lock(&mainInfo->mainMutex);
    mainInfo->completedThisCycle++;
    pthread_cond_signal(&mainInfo->mainCond);
    pthread_mutex_unlock(&mainInfo->mainMutex);

}


string noEOL(const string& st)
{ // Remove eol
  string st1(st);
  string::size_type i = 0;
  while ((i = st1.find(string("\n"))) != string::npos)
    {
      st1.erase(i, 1);
    }
  return st1;
}

  
void PrintAttributes( AttributeList& attributes)
{
  if (attributes.getLength() == 0) return;
  for (unsigned i = 0; i < attributes.getLength(); ++i)
    {
      const XMLCh* name = attributes.getName(i);
      char* c = XMLString::transcode(name);
      const XMLCh* val = attributes.getValue(i);
      char* v = XMLString::transcode(val);
      cout << "Attribute " << c << " value " << v << endl;
      XMLString::release(&c);
      XMLString::release(&v);
    }
}

void* XMLThread(void* p)
{
  // Create and configure the SAX Parser
  unsigned long longInd = (unsigned long)p;
  unsigned myInd = (unsigned)longInd;
  AnimParser* parser = new AnimParser(myInd, mainInfo);
  parser->model = mainInfo->model;
  parser->xmlFileName = mainInfo->threadInfo[myInd].fn;
  parser->setValidationScheme(SAXParser::Val_Never);
  parser->setDoNamespaces(false);
  parser->setDoSchema(false);
  parser->setValidationSchemaFullChecking(false);

  // Create our document handler
  AnimDocHandlers handler (mainInfo);
  parser->setDocumentHandler(&handler);
  parser->setErrorHandler(&handler);
  handler.parser = parser;
  //LocalFileInputSource lf(XMLString::transcode(parser->xmlFileName.c_str()));
  PartialFileInputSource is(
                            XMLString::transcode(parser->xmlFileName.c_str()),
                            myInd,mainInfo);
  // Parse the file
  parser->parse(is);
  return 0;
}

void StartXML ()
{
    static int xmlstarted = 0;
    if (xmlstarted)
    {
        QMessageBox msg;
        msg.setText("XML Already loaded. Reloading is not yet supported");
        msg.exec();
        return;
    }
    for (int i = 0; i < mainInfo->XMLFileList.size(); ++i)
      {
          ThreadInfo *ti = new ThreadInfo();
          mainInfo->threadInfo.push_back(*ti);
      }

    // Lock the main mutex, unlocked when waiting for signals
    pthread_mutex_lock(&mainInfo->mainMutex);

    if (!mainInfo->IsXMLPlatformInitialized())
    {
        // Initialize the xerces system
        XMLPlatformUtils::Initialize();
        mainInfo->SetXMLPlatformInitialized();
    }

    // We loop over the various xml files here and call the
    // parsing thread repeatedy in separate threads.
    // First create the thread control blocks for each thread


    xmlstarted++;
    for (int i = 0; i < mainInfo->XMLFileList.size(); ++i)
      {
        mainInfo->threadInfo[i].fn = mainInfo->XMLFileList[i].toStdString ();
        // Create the xml thread
        pthread_t pt;
        int k = pthread_create(&pt, 0, XMLThread, (void*)i);
        if (k)
          {
            cout << "pthread create failed, error " << k << endl;
            perror("thread create failed ");
            exit(1);
          }
         pthread_cond_wait(&mainInfo->mainCond, &mainInfo->mainMutex);
        //cout << "Thread " << thisInd << " initialized " << endl;
      }

    if (!mainInfo->XMLFileList.empty())
        mainInfo->view->StartAnimation(false);
}

void SetSimulation ()
{


}


int main(int argc, char** argv)
{

  // Allocate the mainInfo structure
  mainInfo = new MainInfo;
  mainInfo->setArgs(argc,argv);
#ifdef WITH_NS3
  Simulation s(mainInfo);
  mainInfo->SetSimulation(&s);
#endif
  // Qt initialization
  QApplication a(argc, argv);
  MainWindow mw(&a,mainInfo);

  mainInfo->setMainWindow(&mw);
  mainInfo->model = mainInfo->view->GetNetModel();

  /*
  for (int i = 1; i < argc; ++i)
    {
      mainInfo->threadInfo.push_back(ThreadInfo());
      mainInfo->XMLFileList.append(argv[i]);
    }
*/
  mainInfo->view->show();
  mainInfo->view->SetUpdateInterval(0.001);
  mainInfo->view->SetViewSize(512, 512);



  mw.setGeometry((int)(QApplication::desktop()->width() -
  (QApplication::desktop()->width() -
    (QApplication::desktop()->width() / 2)) * 1.5) / 2,
    (int)(QApplication::desktop()->height() -
  (QApplication::desktop()->height() -
    (QApplication::desktop()->height() / 2)) * 1.5) / 2,
    (int)((QApplication::desktop()->width() -
  (QApplication::desktop()->width() / 2)) * 1.5),
    (int)((QApplication::desktop()->height() -
  (QApplication::desktop()->height() / 2)) * 1.5));

  a.connect(&a, SIGNAL(lastWindowClosed()), &a, SLOT(quit()));



  if (!mainInfo->XMLFileList.empty())
  {
      StartXML ();
      mainInfo->view->StartAnimation(false);
  }
  a.exec();

}

