/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: George F. Riley<riley@ece.gatech.edu>
 */
// Implementation of the Network Animation Viewer Model

#include <iostream>
#include <math.h>

#include "NetModel.h"
#include "MainWindow/mainwindow.h"
#include "MainWindow/statusbar.h"

using namespace std;
namespace netanim {


// NetModel methods

// Constructor
NetModel::NetModel(MainInfo *mi) : view(nil), mainInfo (mi)
{
  pthread_mutex_init(&nodeMutex, 0);
  pthread_mutex_init(&linkMutex, 0);
  pthread_mutex_init(&packetMutex, 0);
  
}

void NetModel::SetView(NetView* v)
{
  view = v;
}

// Static methods
void NetModel::AddNode(const Node& n)
{
  pthread_mutex_lock(&nodeMutex);
  QPointF topLeft = minMaxCoord.topLeft();
  QPointF bottomRight = minMaxCoord.bottomRight();
  QPointF currentPosition = n.GetLoc2d(0);
#ifdef REMOVE_THIS
  double x = currentPosition.x();
  double y = currentPosition.y();
  // This code automatically re-sizes the view area to accomodate
  // all nodes.  Removed per instructions from RAP, Feb 26, 2010
  // If nodes container empty, always set this value
  if (x < topLeft.x() || nodes.empty())  topLeft.setX(x);
  if (y < topLeft.y() || nodes.empty())  topLeft.setY(y);
  if (x > bottomRight.x() || nodes.empty()) bottomRight.setX(x);
  if (y > bottomRight.y() || nodes.empty()) bottomRight.setY(y);
  minMaxCoord = QRectF(topLeft, bottomRight);
#else
  minMaxCoord = defaultMinMax;
#endif
  nodes[KeyPair_t(n.lpId, n.id)] = n;
  pthread_mutex_unlock(&nodeMutex);
}

// Not a reference return, since other threads might be adding
// and this might become invalid
Node NetModel::GetNode(LpId_t lp, NodeId_t id)
{
  pthread_mutex_lock(&nodeMutex);
  Node n = nodes[KeyPair_t(lp, id)];
  pthread_mutex_unlock(&nodeMutex);
  return n;
}

QRectF NetModel::GetBoundingRect()
{
  pthread_mutex_lock(&nodeMutex);
  QRectF br(minMaxCoord.topLeft(), QPointF());
  // Add 10% borders
  double w = minMaxCoord.width();
  double h = minMaxCoord.height();
  //br.setWidth (w * 1.1);
  //br.setHeight(h * 1.1);
  br.setWidth (w * 1.0);
  br.setHeight(h * 1.0); // No need for margin here
  //br.moveTopLeft(minMaxCoord.topLeft() - QPointF(w * 0.05, h * 0.05));
  br.moveTopLeft(minMaxCoord.topLeft());
  pthread_mutex_unlock(&nodeMutex);
  return br;
}

QRect NetModel::GetViewRect()
{ // Same as the bounding rect, but in integer coordinates
  pthread_mutex_lock(&nodeMutex);
  double w = minMaxCoord.width();
  double h = minMaxCoord.height();
  // if width is < than the initial width on the <topology> node,
  // use the initial.  Ditto for height
  // debug..remove this for testing
  //if (w < defaultMinMax.width())  w = defaultMinMax.width();
  //if (h < defaultMinMax.height()) h = defaultMinMax.height();
  // In order to keep the same x/y aspect ratio, we need to make
  // sure that the ViewRect width == viewRect height
  double extraXMargin = 0;
  double extraYMargin = 0;
  if (w > h)
    {
      extraYMargin = (w - h) / 2;
      h = w;
    }
  else
    {
      extraXMargin = (h - w) / 2;
      w = h;
    }

  double x = minMaxCoord.topLeft().x();
  double y = minMaxCoord.topLeft().y();
  double mx = w * 0.1;  // 10% margins
  double my = h * 0.1;
  //QRect ret(QPoint(floor(x-mx-extraXMargin), floor(y-my-extraYMargin)),
  //          QPoint(ceil(x+w+mx), ceil(y+h+my)));
  QRect ret(QPoint(floor(x-mx/2-extraXMargin), floor(y-my/2-extraYMargin)),
            QSize(ceil(w+mx), ceil(h+my)));
  pthread_mutex_unlock(&nodeMutex);
  return ret;
}

void NetModel::AddPacket(const Packet& p)
{

  // Find the latest lbRx value and use that for the index
  Time_t last = 0;
  for (unsigned i = 0; i < p.rxVec.size(); ++i)
    {
      if (p.rxVec[i].lbRx > last) last = p.rxVec[i].lbRx;
    }
  pthread_mutex_lock(&packetMutex);
  packets.insert(PktMap_t::value_type(last, p));
 // mainInfo->getMainWindow()->getStatusBar()->SetPacketCount(packets.size());

  // Sorting on firstbit tx seems better, right? No?
  //packets.insert(PktMap_t::value_type(p.fbTx, p));
  pthread_mutex_unlock(&packetMutex);
}

void NetModel::AddLink(const WiredLink& wl)
{
  pthread_mutex_lock(&linkMutex);
  links.push_back(wl);
  pthread_mutex_unlock(&linkMutex);
}

void NetModel::AddMobility(LpId_t lp, NodeId_t id, const Mobility& m)
{
  pthread_mutex_lock(&nodeMutex);
  Node& n = nodes[KeyPair_t(lp, id)];
  n.AddMobility(m);
  pthread_mutex_unlock(&nodeMutex);
}

// Accessors
NodeMap_t& NetModel::GetNodes()
{
  pthread_mutex_lock(&nodeMutex);
  return nodes;
}

void NetModel::ReleaseNodes()
{
  pthread_mutex_unlock(&nodeMutex);
}
  
PktMap_t& NetModel::GetPackets()
{
  pthread_mutex_lock(&packetMutex);
  return packets;
}

void NetModel::ReleasePackets()
{
  pthread_mutex_unlock(&packetMutex);
}
  
LinkVec_t& NetModel::GetLinks()
{
  pthread_mutex_lock(&linkMutex);
  return links;
}

void NetModel::ReleaseLinks()
{
  pthread_mutex_unlock(&linkMutex);
}
  




  
} //namespace netanim
