/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: George F. Riley <riley@ece.gatech.edu>
 * Author: John Abraham <john.abraham@gatech.edu>
 */

// Network Animation View

#include <iostream>
#include <iomanip>
#include <sstream>
#include <math.h>

#include <QtGui>
#include <QApplication>
#include <QTimer>
#include <QPainter>
#include <QPaintEvent>
#include <QResizeEvent>
#include <QSize>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QPushButton>
#include <QLCDNumber>
#include <QSlider>
#include <QLabel>
#include <QScrollArea>

#include "NetView.h"
#include "MainInfo/MainInfo.h"
#include "MainWindow/mainwindow.h"
#include "MainWindow/statusbar.h"
#include "XML/AnimOnlineParser.h"
#include "XML/AnimOnlineDocHandler.h"
#include <xercesc/framework/MemBufInputSource.hpp>

#ifdef WITH_NS3
#include "ns3/core-module.h"
#endif


using namespace std;
using namespace xercesc;

namespace netanim {

#define HIST_SCALE 1000.0

extern MainInfo* mainInfo; // Declared and initialized in main

// try this
class MySlider : public QSlider
{
public:
  MySlider(Qt::Orientation o) : QSlider(o)
  {
  }
  void mousePressEvent(QMouseEvent*);
  void mouseMoveEvent(QMouseEvent*);
};


void  MySlider::mousePressEvent(QMouseEvent* me)
{
  if (me->type() == QEvent::MouseButtonPress)
    {
      //if (me->button() == Qt::LeftButton)
      //  cout << "Slider mouse pressed left" << endl;
      //if (me->button() == Qt::RightButton)
      //  cout << "Slider mouse pressed right" << endl;
    }
  QSlider::mousePressEvent(me);
}




void  MySlider::mouseMoveEvent(QMouseEvent* me)
{
  //cout << "Slider mouse moved" << endl;
  QSlider::mouseMoveEvent(me);
}

NetView::NetView(NetModel* m0, QWidget* mw, MainInfo *mi)
  : w(1024), h(1024), minW(w), minH(h), pmW(w), pmH(h),
    paintPM(nil), m(m0), mainWidget(mw), sa(nil),
    sliderMin(0), sliderMax(0),
    simTimeWidget(nil), updateRate(nil), backupSlider(nil),
    updateInterval(0.001), wirelessUpdateInterval(0.0), showWirelessCircles(true), mousePressed(false),
    forward(true),
    firstDisplayedKnown(false),lastDisplayedKnown(false), highestAdvancedTime(0), currentTime(0),
    maxHistoryTime(1.0), minHistoryTime(0),
    linkWidth(1), paused(true), framesPerSecond(24),
    logicalToPixelScale(0.0), pixelToLogicalScale(0.0), zoomScale(1.0),
    endOfData(false), inUpdate(false), stalled(false), trackAll(false), PacketOn(true), mainInfo (mi),NodeIdFontSize(8),ShowNodeId(true),
    AnimationStopped(false)
  #ifdef WITH_NS3
  ,animOnlineInitComplete(false),AnimOnlineThreadIndex(0)
  #endif
{
  timer = new QTimer();
  connect(timer, SIGNAL(timeout()), this, SLOT(TimeToUpdate()));
  recordTimer = new QTimer();
  QObject::connect(recordTimer, SIGNAL(timeout()), this, SLOT(TimeToRecord()));
  
  updateRates[0]  = 0.000001;
  updateRates[1]  = 0.000002;
  updateRates[2]  = 0.000004;
  updateRates[3]  = 0.000008;
  updateRates[4]  = 0.000016;
  updateRates[5]  = 0.000032;
  updateRates[6]  = 0.000064;
  updateRates[7]  = 0.000125;
  updateRates[8]  = 0.000250;
  updateRates[9]  = 0.000500;
  updateRates[10] = 0.001000;
  updateRates[11] = 0.002000;
  updateRates[12] = 0.004000;
  updateRates[13] = 0.008000;
  updateRates[14] = 0.016000;
  updateRates[15] = 0.032000;
  updateRates[16] = 0.064000;
  updateRates[17] = 0.125000;
  updateRates[18] = 0.250000;
  updateRates[19] = 0.500000;
  updateRates[20] = 1.000000;

  trackTimer = new QTimer();
  connect(trackTimer, SIGNAL(timeout()),this, SLOT (trackTimerSlot()));
  trackTimer->setInterval(2000);
  trackTimer->start();
  // just debug, try this
  //QApplication::setOverrideCursor(Qt::WaitCursor);

}


void NetView::SetAnimatorStatusBar()
{
    MainWindow * mw = mainInfo->getMainWindow();
    if (!mw)
        return;
    StatusBar * sb = mw->getStatusBar();
    if (sb)
     sb->SetAnimatorStatusBar();
}

void NetView::ResetAnimatorStatusBar()
{
    MainWindow * mw = mainInfo->getMainWindow();
    if (!mw)
        return;
    StatusBar * sb = mw->getStatusBar();
    if (sb)
    sb->ResetAnimatorStatusBar();

}

QString NetView::GetTabName()
{
    return "Animator";
}

QWidget * NetView::getCentralWidget()
{

    return centralWidget;
}

void NetView::SetUpdateInterval(Time_t ui, bool moveSlider)
{
  if (forward)
    updateInterval = ui;
  else
    updateInterval = -ui;
  
  if (timer && !paused && !inUpdate)
    {
      timer->stop();
      int ms = (int)(ui * 1000.0);
      if (ms == 0) ms = 1;
      //cout << "Setting timer interval to " << ms << " ms" << endl;
      timer->setInterval(ms);
      timer->start();
    }
  if (moveSlider)
    {
      int i = 0;      
      for (; i < MAX_SLIDER-1; ++i)
        {
          if (updateRates[i] >= ui) break; // found it
        }
      updateRateSlider->setValue(i);
    }
  ostringstream oss;
  oss.precision(6);
  oss << fixed << setw(8) << "(Update Period)" << ui;
  updateRate->setText(oss.str().c_str());
}

void NetView::StartAnimation(bool unpause)
{
//  trackAll = true;
//  ffwd->setChecked(true);
  WirelessCircles();
  update(); // Show the time zero state
  if (unpause)
    {
      paused = false;
      int ms = (int)(fabs(updateInterval) * 1000.0);
      if (ms == 0) ms = 1;
      timer->setInterval(ms);
      cout << "Setting timer interval to " << ms << " ms" << endl;
      //timer->setInterval(50); // very slow for debugging
      timer->stop();
      timer->start();
    }

}

Time_t NetView::GetMaxTime()
{
  Time_t maxT = 1e10;
  for (unsigned i = 0; i < mainInfo->threadInfo.size(); ++i)
    {
      ThreadInfo& ti = mainInfo->threadInfo[i];
      maxT = min(maxT, ti.maxTime);
    }
  return maxT;
}

Time_t NetView::GetMaxAdvanced()
{
  Time_t maxT = 1e10;
#ifdef WITH_NS3
  if (mainInfo->IsAnimationOnline())
  {
      ThreadInfo& ti = mainInfo->threadInfo[GetAnimOnlineThreadIndex()];
      maxT = min (maxT,ti.advancedTo);
      return maxT;
  }
#endif
  for (unsigned i = 0; i < mainInfo->threadInfo.size(); ++i)
    {
      ThreadInfo& ti = mainInfo->threadInfo[i];
      maxT = min(maxT, ti.advancedTo);
    }


  return maxT;
}

  
void NetView::SetViewSize(int w0, int h0)
{
  minW = w0;
  minH = h0;
  if (w < minW) w = minW;
  if (h < minH) h = minH;
  setMinimumSize(minW, minH);
  if (paintPM)
    {
      delete paintPM;
      paintPM = nil;
    }
}

NetModel* NetView::GetNetModel()
{
  return m;
}


void NetView::SetShowNodeId()
{
    ShowNodeId = true;
    update();

}

void NetView::ResetShowNodeId()
{
    ShowNodeId = false;
    update();


}

void NetView::increaseNodeIdfontSize()
{
  NodeIdFontSize += 0.1;
  update();
}

void NetView::decreaseNodeIdfontSize()
{
  NodeIdFontSize -= 0.1;
  if (NodeIdFontSize == 0)
      NodeIdFontSize = 1;
  update();
}

int NetView::getNodeIdFontSize()
{
   return NodeIdFontSize;
}

bool NetView::IsShowNodeId()
{
    return ShowNodeId;

}

void NetView::UpdateSimTime(Time_t now)
{
  //cout << "Updating simtimewidget to " << now << endl;
  ostringstream oss;
  oss.precision(6);
  oss << fixed << setw(8) << now <<"(Sim Time)";
  simTimeWidget->setText(oss.str().c_str());
  // And update the backup slider to the correct time
  int v = now * HIST_SCALE;
  backupSlider->setValue(v);
}

// Inherited from QWidget
void NetView::paintEvent(QPaintEvent*)
{
  if (!paintPM)
    { // Should not happen
      //cout << "Creating new pixmap" << endl;
      paintPM = new QPixmap(pmW, pmH);
      paintPM->fill();
      //cout << "Resizing main widget, minW " << minW
      //     << " minH " << minH << endl;
      //mainWidget->resize(minW, minH);
    }
  QPainter p(this);
  p.drawPixmap(QPoint(0,0), *paintPM);
}

void NetView::resizeEvent(QResizeEvent* rev)
{
  //if (rev->size().width()  > minW) minW = rev->size().width();
  //if (rev->size().height() > minH) minH = rev->size().height();
  //cout << "Hello from resize event, w " << rev->size().width()
 //      << " h " << rev->size().height() << endl;
  pmW = rev->size().width();
  pmH = rev->size().height();
  //mw->resize(rev->size());
  UpdateScales();
}

void NetView::mousePressEvent(QMouseEvent* me)
{
  if (me->type() == QEvent::MouseButtonPress)
    {
      if (me->button() == Qt::LeftButton)
        {
          mouseInitialPos = me->pos();
          cout << "Mouse left pressed at " 
               << mouseInitialPos.x() << ", " << mouseInitialPos.y() << endl;
          mouseViewWindow = currentViewWindow;
          mousePressed = true;
        }
    }
  else if (QEvent::MouseButtonRelease)
    {
      mousePressed = false;
    }
}

void NetView::mouseMoveEvent(QMouseEvent* me)
{
  QPoint mouseMovePos = me->pos();
  int deltaX = mouseMovePos.x() - mouseInitialPos.x();
  int deltaY = mouseMovePos.y() - mouseInitialPos.y();
  // deltaX/Y are in viewport coordinates, we need to adjust in window coord
  deltaX *= (double)currentViewWindow.width() / viewPortRect.width();
  deltaY *= (double)currentViewWindow.height() / viewPortRect.height();
  currentViewWindow.moveTo(QPoint(mouseViewWindow.x() - deltaX,
                                  mouseViewWindow.y() - deltaY));
  update();
}

void NetView::update()
{
    if(AnimationStopped)
        return;
  if (!paintPM)
    {
      paintPM = new QPixmap(pmW, pmH);
    }
  paintPM->fill();
  // Draw the nodes, links, and packets here
  QPainter p(paintPM);
  p.setRenderHints(QPainter::Antialiasing);
  QBrush nodeBrush(Qt::black);
  
  QBrush noBrush(Qt::NoBrush);
  QPen blackPen(Qt::black);
  p.setPen(blackPen);
  // First the nodes
  if (currentViewWindow.isEmpty())
    {
      currentViewWindow = m->GetViewRect();
      viewPortRect = p.viewport();
      UpdateScales();
    }
  // try this
  if (pixelToLogicalScale == 0) UpdateScales();
  p.setWindow(currentViewWindow);
  viewPortRect = p.viewport();
  // The node radius is in pixels, so we need to adjust the radius
  // appropriately for the scaled window
  netanim::NodeMap_t& nodes = m->GetNodes();
  for (netanim::NodeMap_t::iterator i = nodes.begin();
       i != nodes.end(); ++i)
    {
      netanim::Node& n = i->second;
      nodeBrush = QBrush(GetColor(Qt::black, n.color));
      ShowNode(p, n, nodeBrush);
    }
  m->ReleaseNodes();
  // Now links
  netanim::LinkVec_t& links = m->GetLinks();
  for (netanim::LinkVec_t::size_type i = 0; i < links.size(); ++i)
    {
      WiredLink& lk = links[i];
      netanim::Node nn1 = m->GetNode(lk.lp1, lk.n1);
      netanim::Node nn2 = m->GetNode(lk.lp2, lk.n2);
      QPointF n1 = nn1.GetLoc2d(currentTime);
      QPointF n2 = nn2.GetLoc2d(currentTime);
      p.drawLine(n1, n2);

    }
  m->ReleaseLinks();

  // Now packets
  // First we reset the wireless speed of light "quick" updates.
  // If any wireless packet is shown during this update, it will
  // be reset
  wirelessUpdateInterval = 0.0;

  PktMap_t& packets = m->GetPackets();
  //cout << "Update to time " << currentTime
  //     << "nPkts " << packets.size() << endl;
  // Here's the approach. For efficiency on each update we do not
  // start looking at the beginning.  Instead, we start with the
  // very first packet displayed on the prior update.  We will likely
  // skip a few of these (if the last bit is now in the past).
  // But this is much more efficient than start from the beginning.
  netanim::PktMap_t::iterator i = packets.begin();
  if (firstDisplayedKnown) i = firstDisplayedLastUpdate;
  bool firstThisUpdate = true;

  if (lastDisplayedKnown && trackAll)
    {
       i = lastDisplayedLastUpdate;
       i++;
   }


  for ( ; i != packets.end(); ++i)
    {
      Packet& pkt = i->second;
      // Break if fbTx in future
      if (pkt.fbTx > currentTime) break;  // fbtx in future, all done
      bool shown;
      if (pkt.wired)
        {
          shown = ShowWiredPacket(pkt, p);
        }
      else
        {
          //Pause(); // for debugging
           if (forward && pkt.forwardshown)
                continue;
          shown = ShowWirelessPacket(pkt, p);
         // std::cout << "fbTx,lbTx:" << pkt.fbTx << ","
        //            << pkt.lbTx << " Curr Time:" << currentTime << std::endl;
        //  std::cout << "fbrx,lbrx:" << pkt.rxVec[0].fbRx
        //            << pkt.rxVec[0].lbRx << std::endl;
          if (forward && shown)
              pkt.forwardshown = true;

        }
      if ((shown && firstThisUpdate) || (trackAll))
        {

            if (trackAll)
                currentTime = max (currentTime,pkt.fbTx);

          firstDisplayedLastUpdate = i;
          firstThisUpdate = false;
          firstDisplayedKnown = true;
          lastDisplayedLastUpdate = i;
          lastDisplayedKnown = true;
        }
    }
  m->ReleasePackets();
  QWidget::update();
}
  

void NetView::show()
{
  //mw->show();
  QWidget::show();
}

void NetView::UpdateScales()
{
  double maxWinWH = max(currentViewWindow.width(), currentViewWindow.height());
  double maxViewWH = max(viewPortRect.width(), viewPortRect.height());
  logicalToPixelScale = maxViewWH / maxWinWH;
  pixelToLogicalScale = maxWinWH  / maxViewWH;
  // And calculate the zoom scale
  NetModel* m = GetNetModel();
  QRectF defaultView = m->GetViewRect();
  zoomScale = defaultView.width() / maxWinWH;
}

bool NetView::ShowWiredPacket(const netanim::Packet& pkt, QPainter& p)
{ // Returns true if packet actually displayed
  // Only display the packet if lbRx in future
  Time_t lbRx = pkt.rxVec[0].lbRx;

  if (!trackAll && (lbRx <= currentTime)) return false; // No longer visible
  // Actively visible at this time
  ShowPacket(pkt, p, pkt.rxVec[0]);
  return true;
}

bool NetView::ShowWirelessPacket(const netanim::Packet& pkt, QPainter& p)
{ // Returns true if packet actually displayed
  // Only display the packet if lbRx in future
  bool   r = false;
  bool   wirelessAnim = false;
  for (unsigned i = 0; i < pkt.rxVec.size(); ++i)
    { // Display to each receiver
      Time_t lbRx = pkt.rxVec[i].lbRx;
      if (lbRx <= currentTime)
      {
          //std::cout<<"Skipping: lbRx:" << lbRx << " currentTime:" << currentTime <<std::endl;
         // continue; // No longer visible
      }
      // Actively visible at this time
      //cout << "Showing wpacket time " << currentTime << endl;
      ShowPacket(pkt, p, pkt.rxVec[i]);
      r = true;  // At least one displayed
      if ( pkt.lbTx < currentTime  && // last bit has transmitted
          pkt.rxVec[i].lbRx > currentTime  || // last bit not yet received
          pkt.fbTx < currentTime &&  // first bit has been transmitted
          pkt.rxVec[i].fbRx > currentTime)    // first bit not yet received
        {
          wirelessAnim = true;
        }
        wirelessAnim = true;
    }
  if (r)
    { // we do this even if !showWirelessircles, as we need some of this info
      // to calcualte the wirelessUpdateInterval.  We of course
      // do not actually draw the circles unless show is true
      p.save();
      QBrush noBrush(Qt::NoBrush);
      p.setBrush(noBrush);
      netanim::Node n1 = m->GetNode(pkt.txLp, pkt.txId);
      QPointF n1Loc = n1.GetLoc2d(currentTime);
      double  maxDist = 0;
      int     maxDistInd = 0;
      for (unsigned i = 0; i < pkt.rxVec.size(); ++i)
        { // Find farthest receiver
          netanim::Node n2 = m->GetNode(pkt.rxVec[i].rxLp, pkt.rxVec[i].rxId);
          QPointF n2Loc = n2.GetLoc2d(currentTime);
          // Compute angle to n2
          double dy = n2Loc.y() - n1Loc.y();
          double dx = n2Loc.x() - n1Loc.x();
          // Compute distance from tx to rx
          double dist = sqrt(dy * dy + dx * dx);
          if (dist > maxDist)
            {
              maxDist = dist;
              maxDistInd = i;
            }
        }
      // Now divide the time from tx to rx in 10 units
      double delayTime = pkt.rxVec[maxDistInd].fbRx - pkt.fbTx;
      double deltaTime = delayTime / 10.0;
      double deltaDist = maxDist / 10.0;
      double totalTime = pkt.fbTx;
      double totalDist = 0;
      double endTime = pkt.rxVec[maxDistInd].lbRx;
      double lastBitDistance = 0;

      // Now set the update interval to the deltaTime to force animatin
      // update more often to see the wireless tx
      // But only if the wireless tx is actuall doing something
      if (wirelessAnim)
        {
          wirelessUpdateInterval = max(deltaTime,0.0001);//0.0001 prevents xml parser lockout
          
          //cout << "WUI " << wirelessUpdateInterval << endl;
        }
      
      if (pkt.lbTx < currentTime)
        { // Last bit has past, see how far it travelled
          lastBitDistance = maxDist * (currentTime - pkt.lbTx) / delayTime;
        }
      
      if (showWirelessCircles && PacketOn)
        {
          p.translate(n1Loc);
          QPen pen(GetColor(Qt::black, pkt.color));
          p.setPen(pen);
          while(totalDist <= maxDist && ((totalTime < currentTime) || (trackAll)))
          {
              if (totalDist > 0)
                {
                  //if (totalTime < endTime && lastBitDistance < totalDist)
                      double opacity = 1.0 - totalDist / maxDist;
                      if (opacity < 0.1) opacity = 0.1;
                      p.setOpacity(opacity);
                      p.drawEllipse(QPointF(), totalDist, totalDist);
                }
                else
                {
                  if ((totalTime >= endTime) &&trackAll)
                      break;
                }
              totalDist += deltaDist;
              totalTime += deltaTime;
            }
        }
      p.restore();
    }
  return r;
}

void NetView::ShowPacket(const netanim::Packet& pkt, QPainter& p, const netanim::PacketRx& rx)
{
    if(!PacketOn)
        return;
  netanim::LpId_t   rxLp = rx.rxLp;
  NodeId_t rxId = rx.rxId;
  Time_t   fbRx = rx.fbRx;
  Time_t   lbRx = rx.lbRx;
  netanim::Node n1 = m->GetNode(pkt.txLp, pkt.txId);
  netanim::Node n2 = m->GetNode(rxLp, rxId);
  QPointF n1Loc = n1.GetLoc2d(currentTime);
  QPointF n2Loc = n2.GetLoc2d(currentTime);

  //std::cout<<"n1Loc x:"<<n1Loc.x()<<" n1Loc y"<<n1Loc.y()<<std::endl;

  // Set origin to n1
  p.save();
  p.translate(n1Loc);
  // Compute angle to n2
  double dy = n2Loc.y() - n1Loc.y();
  double dx = n2Loc.x() - n1Loc.x();
  double angle = atan2(dy, dx) * 360.0 / (M_PI * 2);;
  p.rotate(angle);
      
  // Compute distance from tx to rx
  double dist = sqrt(dy * dy + dx * dx);
  // Compute packet transmission time
  Time_t txTime = pkt.lbTx - pkt.fbTx;
  // Adujst if last bit not yet transmitted
   if (pkt.wired && (pkt.lbTx > currentTime))
    {
      txTime = currentTime - pkt.fbTx;
    }
  
  // Adjust if first bit already received
  if (pkt.wired)
  if (fbRx < currentTime) txTime -= (currentTime - fbRx);
  // Propagation delay on wire
  Time_t delay = fbRx - pkt.fbTx;
  // Compute last bit distance from transmitter
  Time_t lbTime = currentTime - pkt.lbTx; // Relative to current time
  if (trackAll)
      lbTime = lbRx - pkt.fbTx;
  if (lbTime < 0) lbTime = 0;
  double pktWidth = dist * txTime / delay;
  if (delay == 0) pktWidth = dist; //  ?? Is this right?
  // WIRELESS_FIX if (trackAll &&
  if ( (pktWidth > dist))
      pktWidth = dist;
  if (pktWidth > 4) pktWidth -= 4; // Some spacing between pkts
  double pktDist = dist * lbTime / delay;
  if (delay == 0) pktDist = 0; // ?? Is this right?
  
  if (!pkt.wired) pktDist = 0 ;

  //double maxWinWH = max(currentViewWindow.width(),
  //                      currentViewWindow.height());
  //double maxViewWH = max(viewPortRect.width(),
  //                       viewPortRect.height());
  double scaledRadius = pixelToLogicalScale * n1.radius;
#ifdef VERBOSE
  cout << "n2Loc.x " << n2Loc.x() << " n1loc.x " << n1Loc.x()
       << " n2Loc.y " << n2Loc.y() << " n1Loc.y " << n1Loc.y()
       << " winwh " << maxWinWH
       << " viewwh " << maxViewWH
       << " scaled radius " << scaledRadius
       << " zoomscale " << zoomScale
       << endl;
  cout << "Rotatation angle is " << angle << " degrees " << endl;
#endif
  // Reduce width slightly to handle consecutive packets
  double onePix = pixelToLogicalScale;
  if (onePix > pktWidth) onePix = 0;
  QRectF pktRect(QPointF(0,0), QPointF(pktWidth - onePix, scaledRadius * zoomScale));
  //pktRect.moveTopLeft(QPointF(pktDist, -scaledRadius * zoomScale / 2));
  //pktRect.moveTopLeft(QPointF(pktDist, -scaledRadius * zoomScale));
  pktRect.moveTopLeft(QPointF(pktDist, 0));
  // Now configure the painter
  QPen blackPen(Qt::black);
  p.setPen(blackPen);
  p.setOpacity(0.7); // Allows the wire to show through a bit
  // default color blue (arbitrary);
  QBrush brush = QBrush(GetColor(Qt::blue, pkt.color));
  p.setBrush(brush);
  p.drawRect(pktRect);
  //cout << "drawing rect, w " << pktRect.width()
  //     << " height " << pktRect.height() << endl;
  // If a wireless packet, and if the first bit has reached the destination
  // we re-color the node as specified in the rx color value
  p.restore();
  if (!pkt.wired)
    {
        if ((fbRx < currentTime) || trackAll)
        {
          ShowNode(p, n2, QBrush(GetColor(Qt::black, rx.color)));
        }
    }
}

void NetView::ShowNode(QPainter& p, netanim::Node& n, const QBrush& brush)
{
  double scaledRadius = pixelToLogicalScale * n.radius;
  QRectF nodeRect(QPointF(-scaledRadius, -scaledRadius),
                  QPointF(scaledRadius, scaledRadius));
  p.save();
  p.setBrush(brush);
  QPointF center = n.GetLoc2d(currentTime);
  // Use translation instead
  p.translate(center);
  p.scale(zoomScale, zoomScale);
  //nodeRect.moveTopLeft(center);
  if (n.id >= 0 && n.id <=500)
    {
       QFont f;
       if (getNodeIdFontSize () >  m->minMaxCoord.bottomRight().x()/2 )
           f.setPointSizeF(std::min((double)getNodeIdFontSize(),(double) m->minMaxCoord.bottomRight().x()/5));
       else

            f.setPointSizeF(std::min(getNodeIdFontSize() * scaledRadius,(double) m->minMaxCoord.bottomRight().x()/5));
       p.setFont(f);

        if (n.shape == Node::SQUARE)
          {
            p.drawRect(nodeRect);
          }
        else if (n.shape == Node::CIRCLE)
          {
            p.drawEllipse(QPointF(), scaledRadius, scaledRadius);
          }
        if (IsShowNodeId())
        p.drawText(QPointF(),QString::number(n.id));

        p.restore();
        return ;
    }
  if (n.image)
    {
      //double imageOffsetX = n.image->width() *  pixelToLogicalScale / 2.0;
      //double imageOffsetY = n.image->height() * pixelToLogicalScale / 2.0;
      double imageOffsetX = n.image->width()  / 2.0;
      double imageOffsetY = n.image->height() / 2.0;
      p.scale(n.imageScale / zoomScale, n.imageScale/zoomScale); // Is this cumulative?
      p.drawImage(QPointF(-imageOffsetX, -imageOffsetY), *n.image);
    }
  else
    { 
      if (n.shape == Node::SQUARE)
        {
          p.drawRect(nodeRect);
        }
      else if (n.shape == Node::CIRCLE)
        {
          p.drawEllipse(QPointF(), scaledRadius, scaledRadius);
        }
    }
  p.restore();
}

QColor NetView::GetColor(const QColor& b, const string& color)
{
  QColor ret(b); // default
  if (!color.empty())
    {
      if (color[0] == '0')
        { // Color is of the form 0xRRGGBB
          istringstream iss(color.substr(2, string::npos));
          unsigned c = 0;
          iss >> hex >> c;
          int r = (c >> 16) & 0xff;
          int g = (c >> 8 ) & 0xff;
          int b = (c >> 0 ) & 0xff;
          //cout << hex << "r " << r << " g " << g << " b " << b << endl;
          ret = QColor(r, g, b);
        }
      else
        { // Assume just a name
          ret = QColor(color.c_str());
        }
    }
  return ret;
}

// Static methods
// These are pixmaps for the VCR Controls
static const char * const circles_xpm[] = {
"16 16 2 1",
"# c #000000",
". c #ffffff",
"................",
".....######.....",
"....#......##...",
"...#..#####..#..",
"..#.##.....#..#.",
".#..#.####..#.#.",
".#.#.#....#.#..#",
".#.#.#.###.#.#.#",
".#.#.#.#.#.#.#.#",
".#.#.#.##.##.#.#",
".#.#..#..##.#.#.",
"..#.#..###..#.#.",
"..#..##...##.#..",
"...#...###..#...",
"....##....##....",
"......####......"};

  
static const char * const green_circles_xpm[] = {
"16 16 2 1",
"# c #00FF00",
". c #ffffff",
"................",
".....######.....",
"....#......##...",
"...#..#####..#..",
"..#.##.....#..#.",
".#..#.####..#.#.",
".#.#.#....#.#..#",
".#.#.#.###.#.#.#",
".#.#.#.#.#.#.#.#",
".#.#.#.##.##.#.#",
".#.#..#..##.#.#.",
"..#.#..###..#.#.",
"..#..##...##.#..",
"...#...###..#...",
"....##....##....",
"......####......"};

  
static const char * const play_xpm[] = {
"16 16 3 1",
" 	c None",
".	c #FFFFFF",
"+	c #000000",
"                ",
" ++             ",
" ++++           ",
" ++++++         ",
" ++++++++       ",
" ++++++++++     ",
" ++++++++++++   ",
" +++++++++++++  ",
" +++++++++++++  ",
" ++++++++++++   ",
" ++++++++++     ",
" ++++++++       ",
" ++++++         ",
" ++++           ",
" +++            ",
"                "};

static const char * const ffwd_xpm[] = {
"16 16 3 1",
" 	c None",
".	c #FFFFFF",
"+	c #000000",
"                ",
"                ",
"                ",
"                ",
"                ",
" ++++  ++++     ",
"  ++++   ++++   ",
"    ++++   ++++ ",
"   +++++   ++++ ",
"  ++++   ++++   ",
" +++  +++++     ",
"                ",
"                ",
"                ",
"                ",
"                "};


static const char * const green_ffwd_xpm[] = {
"16 16 3 1",
" 	c None",
".	c #FFFFFF",
"+	c #00FF00",
"                ",
"                ",
"                ",
"                ",
"                ",
" +++   +++      ",
"   +++    +++   ",
"     +++   +++  ",
"   +++   +++    ",
" +++   +++      ",
"                ",
"                ",
"                ",
"                ",
"                ",
"                "};

static const char * const red_ffwd_xpm[] = {
"16 16 3 1",
" 	c None",
".	c #FFFFFF",
"+	c #FF0000",
"                ",
"                ",
"                ",
"                ",
"                ",
"  +++   +++     ",
"   +++    +++   ",
"     +++   +++  ",
"   +++    +++   ",
"  +++   +++     ",
"                ",
"                ",
"                ",
"                ",
"                ",
"                "};

static const char * const green_play_xpm[] = {
"16 16 3 1",
" 	c None",
".	c #FFFFFF",
"+	c #00FF00",
"                ",
" ++             ",
" ++++           ",
" ++++++         ",
" ++++++++       ",
" ++++++++++     ",
" ++++++++++++   ",
" +++++++++++++  ",
" +++++++++++++  ",
" ++++++++++++   ",
" ++++++++++     ",
" ++++++++       ",
" ++++++         ",
" ++++           ",
" +++            ",
"                "};

static const char * const red_play_xpm[] = {
"16 16 3 1",
" 	c None",
".	c #FFFFFF",
"+	c #FF0000",
"                ",
" ++             ",
" ++++           ",
" ++++++         ",
" ++++++++       ",
" ++++++++++     ",
" ++++++++++++   ",
" +++++++++++++  ",
" +++++++++++++  ",
" ++++++++++++   ",
" ++++++++++     ",
" ++++++++       ",
" ++++++         ",
" ++++           ",
" +++            ",
"                "};

// For reverse play
static const char * const play_rev_xpm[] = {
"16 16 3 1",
" 	c None",
".	c #FFFFFF",
"+	c #000000",
"                ",
"            ++  ",
"          ++++  ",
"        ++++++  ",
"      ++++++++  ",
"    ++++++++++  ",
"  ++++++++++++  ",
" +++++++++++++  ",
" +++++++++++++  ",
"  ++++++++++++ ",
"    ++++++++++ ",
"      ++++++++ ",
"        ++++++ ",
"          ++++ ",
"           +++ ",
"                "};


static const char * const green_play_rev_xpm[] = {
"16 16 3 1",
" 	c None",
".	c #FFFFFF",
"+	c #00FF00",
"                ",
"            ++  ",
"          ++++  ",
"        ++++++  ",
"      ++++++++  ",
"    ++++++++++  ",
"  ++++++++++++  ",
" +++++++++++++  ",
" +++++++++++++  ",
"  ++++++++++++ ",
"    ++++++++++ ",
"      ++++++++ ",
"        ++++++ ",
"          ++++ ",
"           +++ ",
"                "};


static const char * const pause_xpm[] = {
"16 16 3 1",
" 	c None",
".	c #FFFFFF",
"+	c #000000",
"                ",
"   ++++   ++++  ",
"   ++++   ++++  ",
"   ++++   ++++  ",
"   ++++   ++++  ",
"   ++++   ++++  ",
"   ++++   ++++  ",
"   ++++   ++++  ",
"   ++++   ++++  ",
"   ++++   ++++  ",
"   ++++   ++++  ",
"   ++++   ++++  ",
"   ++++   ++++  ",
"   ++++   ++++  ",
"   ++++   ++++  ",
"                "};

static const char * const red_pause_xpm[] = {
"16 16 3 1",
" 	c None",
".	c #FFFFFF",
"+	c #FF0000",
"                ",
"   ++++   ++++  ",
"   ++++   ++++  ",
"   ++++   ++++  ",
"   ++++   ++++  ",
"   ++++   ++++  ",
"   ++++   ++++  ",
"   ++++   ++++  ",
"   ++++   ++++  ",
"   ++++   ++++  ",
"   ++++   ++++  ",
"   ++++   ++++  ",
"   ++++   ++++  ",
"   ++++   ++++  ",
"   ++++   ++++  ",
"                "};

static const char *record_xpm[] = {
/* width height ncolors chars_per_pixel */
"32 32 16 1",
/* colors */
"  c #000000000000",
"! c #800000000000",
"# c #000080000000",
"$ c #800080000000",
"% c #000000008000",
"& c #800000008000",
"' c #000080008000",
"( c #800080008000",
") c #c000c000c000",
"* c #ff0000000000",
"+ c #0000ff000000",
", c #ff00ff000000",
"- c #00000000ff00",
". c #ff000000ff00",
"/ c #0000ff00ff00",
"0 c #ff00ff00ff00",
/* pixels */
"                                ",
" 0000000000000000000000000000000",
"  0         0       000000000000",
" 0 --------- ------- 000000 0000",
" 0 ---------          0000 , 000",
"  0          --------     ,, (00",
" 00 - --------------- - - ,, ((0",
"000 - --------------- - - ,, ((0",
"000 - --------------- - - ,, ((0",
"000 - --------------- - - ,, ((0",
"000 - --------------- - - ,, ((0",
"000 - ---------------     ,, ((0",
"0000  -  ----- - - -- (((( , ((0",
"00000 --------------- ((((( (((0",
"000000               (((0000((00",
"0000000(((((((((((((((((00000000",
"00000000(((((((((((((((000000000",
"0000               0000000000000",
"0000 ,0,0,0,0,0,0, (000000000000",
"0000 0,0,0,0,0,0,0 ((00000000000",
"0000 ,0,0               00000000",
"0000 0, , ,0,0,0,0,0,0, (0000000",
"0000 , 0  0,0,0,0,0,0,0 ((000000",
"0000 0, , ,0 0,0,0,0 0, ((000000",
"0000 ,0,0 0 0 0   0 0 0 ((000000",
"0000 0,0, ,0 0,0,0,0 0, ((000000",
"0000      0,0,0,0,0,0,0 ((000000",
"00000(((( ,0,0,0,0,0,0, ((000000",
"000000((( 0,0,0,0,0,0,0 ((000000",
"000000000               ((000000",
"0000000000((((((((((((((((000000",
"00000000000(((((((((((((((000000"
} ;

static const char * const zoomout_xpm[] = {
"16 16 3 1",
" 	c None",
".	c #FFFFFF",
"+	c #000000",
"                ",
"                ",
"                ",
"                ",
"                ",
"                ",
"                ",
" +++++++++++++  ",
" +++++++++++++  ",
"                ",
"                ",
"                ",
"                ",
"                ",
"                ",
"                "};

static const char * const zoomin_xpm[] = {
"16 16 3 1",
" 	c None",
".	c #FFFFFF",
"+	c #000000",
"                ",
"       ++       ",
"       ++       ",
"       ++       ",
"       ++       ",
"       ++       ",
"       ++       ",
" ++++++++++++++ ",
" ++++++++++++++ ",
"       ++       ",
"       ++       ",
"       ++       ",
"       ++       ",
"       ++       ",
"       ++       ",
"                "};

static const char * const zoomreset_xpm[] = {
"16 16 3 1",
" 	c None",
".	c #FFFFFF",
"+	c #000000",
"                ",
"   ++++++++++   ",
"  ++++++++++++  ",
" ++          ++ ",
" ++          ++ ",
" ++          ++ ",
" ++          ++ ",
" ++          ++ ",
" ++          ++ ",
" ++          ++ ",
" ++          ++ ",
" ++          ++ ",
"  ++         +  ",
"   ++++++++++   ",
"    ++++++++    ",
"                "};




QWidget * NetView::Create(QApplication* app)
{
  mw = new QWidget();
  mainWidget = mw;
  NetModel* model = new NetModel(mainInfo);
  m=model;

  mainInfo->view = this;
  mw->setWindowTitle("Network Animator");

  // Try making layouts
  QVBoxLayout* vbox = new QVBoxLayout();


  // The top portion of NetAnim has two parts (two Hboxes)
  // 1. hbox0 carrying the History Slider,updateRate Slider and labels
  // 2. gridbox carrying the Quit button
  //    and other buttons such as play,pause,zoom (vcr controls)


  // Populate hbox0

  QHBoxLayout* hbox0 = new QHBoxLayout();
  QLabel* HistorySliderLabel = new QLabel("History");
  hbox0->addWidget(HistorySliderLabel);
  hbox0->setAlignment(Qt::AlignHCenter);
  vbox->addLayout(hbox0);
  backupSlider = new MySlider(Qt::Horizontal);
  backupSlider->setRange(0, (int)(maxHistoryTime * HIST_SCALE));
  connect(backupSlider, SIGNAL(sliderMoved(int)), this, SLOT(BackupSliderMoved(int)));

  sliderMin = new QLabel("0.000");
  sliderMax = new QLabel("1.000");


  hbox0->addWidget(sliderMin);
  hbox0->addWidget(backupSlider);
  hbox0->addWidget(sliderMax);
  updateRate = new QLabel("(Update Period)0.000001");
  // Create the slider
  updateRateSlider = new MySlider( Qt::Horizontal);
  updateRateSlider->setRange(0, MAX_SLIDER - 1);
  connect(updateRateSlider, SIGNAL(sliderMoved(int)), this, SLOT(RateSliderMoved(int)));
  // Create the simtime display
  simTimeWidget = new QLabel("0.000000(Sim Time)");
  hbox0->addWidget(updateRate);
  hbox0->addWidget(updateRateSlider);
  hbox0->addWidget(simTimeWidget);


  hbox1 = new QHBoxLayout();
  quitButton = new QPushButton("Quit");
  connect(quitButton, SIGNAL(clicked()), app, SLOT(quit()));
  quitButton->setToolTip("Quit the Application");
  // Put the vcr controls
  circles = new QPushButton( QPixmap( circles_xpm), QString());
  connect(circles, SIGNAL(clicked()), this, SLOT(WirelessCircles()));
  circles->setToolTip("Animates Wireless tx as circles");


  playrev = new QPushButton( QPixmap( play_rev_xpm ), QString());
  connect(playrev, SIGNAL(clicked()), this, SLOT(PlayReverse()));
  playrev->setToolTip("Play the Animation in Reverse");


  play = new QPushButton( QPixmap( play_xpm ), QString());
  connect(play, SIGNAL(clicked()), this, SLOT(Play()));
  play->setToolTip("Play the Animation in the normal forward direction");

  ffwd = new QCheckBox ("Tracking!!");
  ffwd->setChecked(true);
  connect(ffwd, SIGNAL(stateChanged(int)), this, SLOT(FFwdPressed(int)));
  ffwd->setToolTip("Try to track all packets by overriding the Update interval");


  pause = new QPushButton( QPixmap( pause_xpm), QString());
  connect(pause, SIGNAL(clicked()), this, SLOT(Pause()));
  pause->setToolTip("Pause the Animation");

  QPushButton* record = new QPushButton( QPixmap( record_xpm), QString());
  connect(record, SIGNAL(clicked()), this, SLOT(Record()));
  record->setToolTip("Record the Animation");


  QPushButton* zoomin = new QPushButton( QPixmap( zoomin_xpm), QString());
  connect(zoomin, SIGNAL(clicked()), this, SLOT(ZoomIn()));
  zoomin->setToolTip("Zoom in");

  QPushButton* zoomout = new QPushButton( QPixmap( zoomout_xpm), QString());
  connect(zoomout, SIGNAL(clicked()), this, SLOT(ZoomOut()));
  zoomout->setToolTip("Zoom out");


  QPushButton* zoomreset = new QPushButton( QPixmap( zoomreset_xpm), QString());
  connect(zoomreset, SIGNAL(clicked()), this, SLOT(ZoomReset()));
  zoomreset->setToolTip("Zoom Reset");


  packettoggle = new QPushButton("Pkt ON");
  packettoggle->setChecked(true);
  packettoggle->setToolTip("Toggle to view packets or just the topology");
  connect(packettoggle, SIGNAL(clicked()), this, SLOT(PacketToggle()));





  hbox1->addWidget(quitButton);
  hbox1->addWidget(circles);
  hbox1->addWidget(playrev);
  hbox1->addWidget(play);
  //hbox1->addWidget(ffwd);
  hbox1->addWidget(pause);
  hbox1->addWidget(record);
  hbox1->addWidget(zoomin);
  hbox1->addWidget(zoomout);
  hbox1->addWidget(zoomreset);
  hbox1->addWidget(packettoggle);





#ifdef WITH_NS3
  StopAnimationInterfaceButton = new QPushButton("Stop Animation");
  StopAnimationInterfaceButton->setToolTip("Press to Disable Animation completely");
  connect(StopAnimationInterfaceButton, SIGNAL(clicked()),this, SLOT(StopAnimationInterfaceSlot()));
  hbox1->addWidget(StopAnimationInterfaceButton);

  animation_onlineButton = new QCheckBox("Offline");
  animation_onlineButton->setIcon(QIcon(QPixmap(OFFLINEICON)));
  //animation_onlineButton->setCheckable(true);
  animation_onlineButton->setToolTip("Enable this if you want to animation during simulation");
  connect(animation_onlineButton, SIGNAL(stateChanged(int)), this, SLOT(animationOnline_slot(int)));
  hbox1->addWidget(animation_onlineButton);


#endif



  vbox->addLayout(hbox1);
  vbox->addWidget(this);
  vbox->setStretchFactor(this, 1);
  mw->setLayout(vbox);
  // We delay the shows until after the xml file parsed
  //mw->show();
  //v->show();
  //v->resize(1024,1024);

  centralWidget = mw;
  return mw;
}



void NetView::PacketToggle()
{
    PacketOn =!PacketOn;

    if(PacketOn)
    {

        packettoggle->setText("Pkt On");
    }
    else
    {

        packettoggle->setText("Pkt Off");
    }

}

Time_t NetView::GetNextPacketTime ()
{
    Time_t NextPacketTime = currentTime;
    netanim::PktMap_t& packets = m->GetPackets();
    for ( netanim::PktMap_t::iterator i = packets.begin(); i != packets.end(); ++i)
    {
        netanim::Packet& pkt = i->second;
        if (pkt.fbTx > currentTime)
        {
            NextPacketTime = pkt.fbTx;
            cout<<"Found next packet"<<endl;
            break;
        }
    }
    m->ReleasePackets();
    return NextPacketTime;
}

Time_t NetView::GetNextFirstFbTxTime()
{
    //PrintPendingPacketList();

    netanim::PktMap_t& packets = m->GetPackets();
    netanim::PktMap_t::iterator i = packets.begin();
    if (lastDisplayedKnown) {
        i = lastDisplayedLastUpdate;
        i++;
    }
   for ( ; i != packets.end(); ++i)
   {
        netanim::Packet& pkt = i->second;
        m->ReleasePackets();
        return pkt.fbTx;
   }
    m->ReleasePackets();
   return 0;
}


#ifdef WITH_NS3

bool NetView::IsAnimationStopped()
{
    return AnimationStopped;
}
void NetView::StopAnimationInterfaceSlot()
{
    AnimationStopped = !AnimationStopped;
    if(AnimationStopped)
    {
        StopAnimationInterfaceButton->setText("Resume Animation");
        ffwd->setChecked(false);
        timer->stop();
        mainInfo->GetSimulation()->StopAnimationInterface();
        StopAnimationInterfaceButton->setVisible(false);
    }
    else
    {
        //QMessageBox msg;
        //msg.setText("Restart Animation not yet implemented");
        //msg.exec();
        ffwd->setChecked(true);
        timer->start();
        StopAnimationInterfaceButton->setText("Stop Animation");
        mainInfo->GetSimulation()->RestartAnimationInterface();

    }

}

void NetView::focusInEvent(QFocusEvent * f)
{
    std::cout<<"NetView GotFocus"<<std::endl;
}

void NetView::InitAnimOnlineXML()
{
    if (animOnlineInitComplete)
        return;
   if (!mainInfo->IsXMLPlatformInitialized())
        XMLPlatformUtils::Initialize();
  // NetViewTemporaryHack();
   mainInfo->SetXMLPlatformInitialized();
   animOnlineInitComplete = true;

   AnimOnlineThreadIndex = mainInfo->AddNewXMLThread();

   pAnimOnlineParser = new AnimOnlineParser(AnimOnlineThreadIndex, mainInfo);
   pAnimOnlineParser->model = mainInfo->model;
   //parser->xmlFileName = mainInfo->threadInfo[myInd].fn;
   pAnimOnlineParser->setValidationConstraintFatal(false);
   pAnimOnlineParser->setValidationScheme(SAXParser::Val_Never);
   pAnimOnlineParser->setDoNamespaces(false);
   pAnimOnlineParser->setDoSchema(false);
   pAnimOnlineParser->setValidationSchemaFullChecking(false);
   pAnimOnlineParser->setExitOnFirstFatalError(false);
   // Create our document handler
   pAnimOnlineDocHandlers = new AnimOnlineDocHandlers (mainInfo);
   pAnimOnlineParser->setDocumentHandler(pAnimOnlineDocHandlers);
   pAnimOnlineParser->setErrorHandler(pAnimOnlineDocHandlers);
   pAnimOnlineDocHandlers->parser = pAnimOnlineParser;

}

int NetView::GetAnimOnlineThreadIndex()
{
    return AnimOnlineThreadIndex;
}

void NetView::NetViewTemporaryHack()
{
    for (ns3::NodeList::Iterator i= ns3::NodeList::Begin() ;
         i != ns3::NodeList::End();
         i++)
    {
        ns3::Ptr <ns3::Node> n = *i;
        ns3::Vector v ;
        ns3::Ptr <ns3::MobilityModel> mobility = n->GetObject <ns3::MobilityModel> ();
        if (!mobility)
          {
              return;
          }
        else
          {
            v = mobility->GetPosition ();
          }

        stringstream ss;
        string lp0 = "0";
        ss << n->GetId();
        string id0 = ss.str();
        stringstream ss1;
        ss1 << v.x;

        string locx0 = ss1.str();
        stringstream ss2;
        ss2 << v.y;
        string locy0 = ss2.str();


        netanim::Node   na(atol(lp0.c_str()), atol(id0.c_str()),
                 atof(locx0.c_str()), atof(locy0.c_str()), 0);

         na.shape = Node::SQUARE;
         std::cout <<"Node position:"<<v.x<<","<<v.y<<endl;
         std::cout <<atof(locx0.c_str())<<endl;
         m->AddNode(na);


    }

}

void NetView::NotifyAnimationOnline(const char *s)
{
    //std::cout<<"Got String:"<<s<<endl;
    //std::cout<<"End String"<<endl;
    InitAnimOnlineXML();
    MemBufInputSource* memBufIS = new MemBufInputSource
     (
         (const XMLByte *)s
                , static_cast<const XMLSize_t>(strlen(s))
         , "test"
         , false
     );

    pAnimOnlineParser->parse(*memBufIS);
}

void NetView::SetOnlineHBox()
{
    quitButton->hide();
    play->hide();
    pause->hide();
    playrev->hide();
    mainInfo->GetCommonVCR()->show();
    hbox1->insertWidget(0,mainInfo->GetCommonVCR());

}


void NetView::ResetOnlineHBox()
{
    mainInfo->GetCommonVCR()->hide();
    quitButton->show();
    play->show();
    pause->show();
    playrev->show();


}
void NetView::animationOnline_slot(int state)
{
    if (animation_onlineButton->isChecked())
        animationOnline = true;
    else
        animationOnline = false;
    if(animationOnline)
    {
        animation_onlineButton->setIcon(QIcon(QPixmap(ONLINEICON)));
        animation_onlineButton->setText("Online");
        mainInfo->SetAnimationOnline();
        SetOnlineHBox();
        animation_onlineButton->hide();
        mainInfo->getMainWindow()->oncurrentabChanged(0);
    }
    else
    {

        animation_onlineButton->setIcon(QIcon(QPixmap(OFFLINEICON)));
        animation_onlineButton->setText("Offline");
        mainInfo->ResetAnimationOnline();
        ResetOnlineHBox();
        mainInfo->getMainWindow()->oncurrentabChanged(0);

    }
}
#endif

void NetView::trackTimerSlot()
{
    static int colortoggle = 0;
    QPalette pal = ffwd->palette();
    if (colortoggle)
    pal.setColor(QPalette::WindowText,Qt::red);
    else
    pal.setColor(QPalette::WindowText, Qt::black);

    if (ffwd->isChecked())
    {
        ffwd->setPalette(pal);
        ffwd->setText("Tracking!!");
    }
    else
    {
        pal.setColor(QPalette::WindowText, Qt::black);
        ffwd->setPalette(pal);
        ffwd->setText("Not Tracking");

    }

    trackTimer->start();
    colortoggle = !colortoggle;

}
// Slots
void NetView::TimeToUpdate()
{
    // std::cout<<"Current Time:"<<currentTime<<std::endl;
    //Time_t advanceToTime;
    if (inUpdate)
      { // Insure not re-entering while waiting for xml reader
        return;
      }

    inUpdate = true;
    //Time_t originalTime = currentTime;
    Time_t thisInterval = updateInterval;

    if (wirelessUpdateInterval > 0 && wirelessUpdateInterval < updateInterval)
      {
        thisInterval = wirelessUpdateInterval;
      }
    //cout << "Advancing time by " << thisInterval;
    if (forward && endOfData)
      {
        inUpdate = false;
        return;
      }

    //advanceToTime = currentTime + thisInterval;

    Time_t nextTime = GetNextFirstFbTxTime();
    if (trackAll && (nextTime == 0))
    {
      advanceToTime += thisInterval;
    }
    else
    {
     advanceToTime = currentTime + thisInterval;
    }


        if (trackAll)
        {
           currentTime = max (currentTime,nextTime);
        }



    if (!forward)
      { // playing in reverse, back up
        endOfData = false;
        if (advanceToTime < 0) advanceToTime = 0;
        // We also need to start over on looking for packets,
        // since by backing up we are changing the starting point
        // we need a better solution here.... :-(
        firstDisplayedKnown = false;
      }
    if (advanceToTime > GetMaxAdvanced())
      {
        // Here we make sure we aren't advancing beyond the last itme
        // found by any thread that has found the </anim>> tag
        if (advanceToTime > GetMaxTime())
          { // Actually allow this update, but no more
            endOfData = true;
            //currentTime = originalTime;
            //return;
          };
        //cout << "New HTA " << highestTimeAdvanced << endl;
      }
    else
      { // We are in history mode, set the slider appropriately
        //unsigned nsv = (unsigned)(advanceToTime / GetMaxAdvanced() *
        //                          maxHistorySize);
        //cout << "Setting slier value to " << nsv << endl;
        //backupSlider->setValue(nsv);
      }

    //cout << " new time " << currentTime << endl;
    // Tell the parsers how far to advance. Let them go a bit ahead
    //if (advanceToTime > mainInfo->advanceTo)
    if (advanceToTime > GetMaxAdvanced())
      { // Not in history mode, allow parser threads to continue
        //mainInfo->advanceTo = currentTime + thisInterval;
        mainInfo->advanceTo = advanceToTime + 1e-3; // ??
        for (unsigned i = 0; i < mainInfo->threadInfo.size(); ++i)
          {
            ThreadInfo& ti = mainInfo->threadInfo[i];
            // Signal each thread to advance
            pthread_mutex_lock(&ti.lock);
            pthread_cond_signal(&ti.cond);
            pthread_mutex_unlock(&ti.lock);
          }
        // Note we have the mainMutex locked at this point, so we
        // can safely set completetedThisCycle to zero here.
        mainInfo->completedThisCycle = 0;
        // Wait for each thread to complete the advance
        while(mainInfo->completedThisCycle < mainInfo->threadInfo.size()
      #ifdef WITH_NS3
              &&
              !mainInfo->IsAnimationOnline()
      #endif
              )
          {
            //pthread_mutex_lock(&mainInfo->coutMutex);
            //cout << "NetView Processing QT Events" << endl;
            //pthread_mutex_unlock(&mainInfo->coutMutex);
            Time_t saveUpdateInt = updateInterval;

            QCoreApplication::processEvents();

            if (updateInterval != saveUpdateInt)
              { // UpdateInterval changed, recompute advanceToTime
                advanceToTime = currentTime + thisInterval;
                if (!forward)
                  { // playing in reverse, back up
                    endOfData = false;
                    if (advanceToTime < 0) advanceToTime = 0;
                    // We also need to start over on looking for packets,
                    // since by backing up we are changing the starting point
                    // we need a better solution here.... :-(
                    firstDisplayedKnown = false;
                  }
                else
                  {
                    mainInfo->advanceTo = advanceToTime + 1e-3; // ??
                  }
                //cout << "UpdateInt " << updateInterval << endl;
              }
            //pthread_mutex_lock(&mainInfo->coutMutex);
            //cout << "NetView waiting all complete, completed "
            //     <<  mainInfo->completedThisCycle
            //     << " pass " << ++dbcount
            //     << endl;
            //pthread_mutex_unlock(&mainInfo->coutMutex);
            // Wait for one or more threads to signal they are done
#ifdef WITH_NS3
            if(!mainInfo->IsAnimationOnline())
#endif
            pthread_cond_wait(&mainInfo->mainCond, &mainInfo->mainMutex);
            //pthread_mutex_lock(&mainInfo->coutMutex);
            //cout << "NetView got mainCond, completed "
            //     <<  mainInfo->completedThisCycle
            //     << endl;
            //pthread_mutex_unlock(&mainInfo->coutMutex);
          }
      }
    highestAdvancedTime = min(GetMaxAdvanced(), advanceToTime);
    if (highestAdvancedTime > maxHistoryTime)
      {
        maxHistoryTime = highestAdvancedTime;
        ostringstream oss;
        oss.precision(3);
        oss << fixed << setw(5) << maxHistoryTime;
        sliderMax->setText(oss.str().c_str());
        // And set the slider range
        backupSlider->setRange(minHistoryTime * HIST_SCALE,
                               maxHistoryTime * HIST_SCALE);
      }
    if(!trackAll)
    {
       #ifdef WITH_NS3
        if (!animationOnline)
         {
            currentTime += updateInterval;
         }
        else
        {
            currentTime = ns3::Simulator::Now().GetSeconds ();
        }
       #else
            currentTime += updateInterval;
       #endif

    }

   // std::cout<<"current Time:"<<currentTime<<" advanced time:"<<advanceToTime<<std::endl;
    if (forward)
      { // check for stall
        if (highestAdvancedTime >= advanceToTime)
          { // Not stalled
            play->setIcon(QPixmap(green_play_xpm));  // Set to green
            stalled = false;
          }
        else
          {
            play->setIcon(QPixmap(red_play_xpm));  // Set to red
            stalled = true;
          }
      }
    //currentTime = advanceToTime;
    UpdateSimTime(currentTime);
    update();
    inUpdate = false;
}



void NetView::TimeToRecord()
{ // Grab the widget and record here
  QPixmap pixmap = QPixmap::grabWidget(mainWidget);
  static int frameNumber = 0;

  ostringstream oss;
  oss << "recordings/ns3anim";
  oss.width(5);
  oss.fill('0');
  oss << frameNumber++;
  oss << ".png";

  QString qstr = oss.str().c_str();
  if (!pixmap.save(qstr))
  {
      cout<<"Unable to save snapshot"<<endl;
      return;
  }
 // cout << "Saved " << oss.str()
 //      << " size " << pixmap.width()
 //      << " x " << pixmap.height() << endl;
}

  
void NetView::WirelessCircles()
{
  showWirelessCircles = !showWirelessCircles;
  if (showWirelessCircles)
    {
      circles->setIcon(QPixmap(green_circles_xpm));
    }
  else
    {
      circles->setIcon(QPixmap(circles_xpm));
    }
}

void NetView::Play()
{

  StartAnimation(true);
  if (stalled)
    {
      play->setIcon(QPixmap(red_play_xpm));  // Set to red
    }
  else
    {
      play->setIcon(QPixmap(green_play_xpm));  // Set to green
    }
#ifdef WITH_NS3
  if (mainInfo->IsAnimationOnline())
      mainInfo->GetSimulation()->Play();
#endif

  playrev->setIcon(QPixmap(play_rev_xpm)); // Set reverse to normal
  pause->setIcon(QPixmap(pause_xpm));      // Set pause to normal
  
  if (updateInterval < 0) updateInterval = -updateInterval;
  forward = true;
  paused = false;

  int ms = (int)(fabs(updateInterval) * 1000.0);
  if (ms == 0) ms = 1;
  timer->setInterval(ms);
  //cout << "Setting timer interval to " << ms << endl;
  timer->start();
}

void NetView::FFwdPressed(int state)
{

    if (ffwd->isChecked())
    {
        trackAll = true;
        ffwd->setText("Tracking!!");
    }
    else
    {
        trackAll = false;
        ffwd->setText("Not Tracking");
    }

}

void NetView::FFwdReleased()
{


}
void NetView::PlayReverse()
{
  play->setIcon(QPixmap(play_xpm));        // Set play to normal
  playrev->setIcon(QPixmap(green_play_rev_xpm)); // Set reverse to green
  trackAll = false;
  ffwd->setChecked(false);
  pause->setIcon(QPixmap(pause_xpm));      // Set pause to normal
  //cout << "Play Reverse" << endl;
  if (updateInterval > 0) updateInterval = -updateInterval;
  forward = false;
  paused = false;
  endOfData = false;
  int ms = (int)(fabs(updateInterval) * 1000.0);
  if (ms == 0) ms = 1;
  timer->setInterval(ms);
  //cout << "Setting timer interval to " << ms << endl;
  timer->start();
}

void NetView::Pause()
{
  play->setIcon(QPixmap(play_xpm));        // Set play to normal
  playrev->setIcon(QPixmap(play_rev_xpm)); // Set reverse to norlam
  pause->setIcon(QPixmap(red_pause_xpm));      // Set pause to red
  if (paused)
    {
      // If pause clicked when already paused, assume single-step
      TimeToUpdate();
    }
  paused = true;
  timer->stop();
}

void NetView::Record()
{ // not coded yet
  cout << "Start recording here" << endl;
  if (recordTimer->isActive())
    { // Stop recording if active
      recordTimer->stop();
    }
  else
    {
      int msInterval = 1000/framesPerSecond;
      if (msInterval == 0) msInterval = 1;
      recordTimer->setInterval(msInterval);
      recordTimer->start();
    }
}

void NetView::ZoomIn()
{
  int x = currentViewWindow.x();
  int y = currentViewWindow.y();
  int w = currentViewWindow.width();
  int h = currentViewWindow.height();
  currentViewWindow.setX(x + w / 4);
  currentViewWindow.setY(y + h / 4);
  currentViewWindow.setWidth (w / 2);
  currentViewWindow.setHeight(h / 2);
  UpdateScales();
  update();
}

void NetView::ZoomOut()
{
  int x = currentViewWindow.x();
  int y = currentViewWindow.y();
  int w = currentViewWindow.width();
  int h = currentViewWindow.height();
  currentViewWindow.setX(x - w/2);
  currentViewWindow.setY(y - h/2);
  currentViewWindow.setWidth(w * 2);
  currentViewWindow.setHeight(h * 2);
  UpdateScales();
  update();
}

void NetView::ZoomReset()
{
  NetModel* m = GetNetModel();
  currentViewWindow = m->GetViewRect();
  UpdateScales();
  update();
}

  
void NetView::BackupSliderMoved(int value)
{
  // Set paused state on any backup slider movement
  play->setIcon(QPixmap(play_xpm));        // Set play to normal
  playrev->setIcon(QPixmap(play_rev_xpm)); // Set reverse to norlam
  pause->setIcon(QPixmap(red_pause_xpm));      // Set pause to red
  paused = true;
  timer->stop();
  endOfData = false;
  stalled = false;
  Time_t histTime = value / HIST_SCALE;
  //cout << "Backup slider to time " << histTime << endl;
  if (histTime > highestAdvancedTime)
    { // Can't move that way!
      int v = currentTime * HIST_SCALE;
      backupSlider->setValue(v);
    }
  else
    { // Calculate the time of this slider value and update at that
      // time.
      currentTime = value / HIST_SCALE;
      // We also need to start over on looking for packets,
      // since by backing up we are changing the starting point
      firstDisplayedKnown = false;
      UpdateSimTime(currentTime);
      update();  // Display the prior state
    }
}

void NetView::RateSliderMoved(int value)
{
  SetUpdateInterval(updateRates[value], false);
}
} //namespace netanim
