/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: George F. Riley<riley@ece.gatech.edu>
 */
// Main window for the Network Animation Viewer

#ifndef __ANIM_VIEW_H__
#define __ANIM_VIEW_H__
#include <vector>
#include <deque>

#include <QWidget>
#include <QMainWindow>
#include "MainWindow/mainwindow.h"
#include "AnimTab/animtab.h"
#include "Animator/NetModel.h"
#include "XML/AnimOnlineParser.h"
#include "XML/AnimOnlineDocHandler.h"


namespace netanim {


#define MAX_SLIDER 21
#define ONLINEICON "images/connected.png"
#define OFFLINEICON "images/disconnected.png"

class NetView : public QWidget
{
  Q_OBJECT
public:
  NetView(NetModel*, QWidget*, MainInfo *);
  void SetUpdateInterval(Time_t, bool = true);
  void StartAnimation(bool);
  Time_t GetMaxTime();        // Returns minimum "maxTime" for each thread
  Time_t GetMaxAdvanced();    // Returns minimum "advancedTo" for each thread
  void SetViewSize(int w, int h);
  NetModel* GetNetModel();
  void UpdateSimTime(Time_t);
  //Inherited from QWidget
  void paintEvent(QPaintEvent*);
  void resizeEvent(QResizeEvent*);
  void mousePressEvent(QMouseEvent*);
  void mouseMoveEvent(QMouseEvent*);
  void update();
  void show();              // Override show so we can show the main window also
  //void UpdateWithHistory(); // Update and add this screen to history buffer
  void UpdateScales();      // Update the logical/pixel scale factors
  bool ShowWiredPacket(const netanim::Packet&, QPainter&);
  bool ShowWirelessPacket(const netanim::Packet&, QPainter&);
  void ShowPacket(const netanim::Packet&, QPainter&, const netanim::PacketRx&);
  void ShowNode(QPainter&, netanim::Node&, const QBrush&);
  QColor GetColor(const QColor&, const std::string& );  // Creates the specified color
  Time_t GetNextPacketTime ();
  Time_t GetNextFirstFbTxTime ();
  QWidget * getCentralWidget();
  QString GetTabName();

public:
  int w;                    // Current width
  int h;                    // Current height
  int minW;                 // Minimum width
  int minH;                 // Minimum height
  int pmW;                  // pixmap width
  int pmH;                  // pixmap height
  QPixmap*     paintPM;     // The pixmap to paint
  //int          historyIndex;// If >= 0, display from history
  NetModel*    m;           // The corresponding model
  QWidget*     mainWidget;  // Used in saving the png files for movies
  QScrollArea* sa;     
  QTimer*      timer;
  QTimer*      trackTimer;
  QTimer*      recordTimer;
  QLabel*      sliderMin;
  QLabel*      sliderMax;
  QLabel*      simTimeWidget;
  QLabel*      updateRate;
  QSlider*     backupSlider;
  QSlider*     updateRateSlider;
  QPushButton* playrev;
  QPushButton* play;
  QCheckBox * ffwd;
  QPushButton* pause;
  QPushButton* circles;
  QPointer <QPushButton>  packettoggle;
  QPointer <QCheckBox> animation_onlineButton;
  QPointer <QPushButton> StopAnimationInterfaceButton;

  Time_t       updateInterval;
  Time_t       wirelessUpdateInterval; // If non-zero, we are animating wireless with circles
  bool         showWirelessCircles;    // True if wireless circles desired
  bool         mousePressed;
  QPoint       mouseInitialPos;     // Mouse position at last click
  QRect        currentViewWindow;
  QRect        mouseViewWindow;     // View window at last mouse clock
  QRect        viewPortRect;        // Viewport from last paint
  // This is the integer range of the back upslider
  //unsigned     maxHistorySize;
  //unsigned     updateNumber;        // Maximum update counter
  bool         forward;             // True if advancing forward
  netanim::PktMap_t::iterator firstDisplayedLastUpdate;
  netanim::PktMap_t::iterator lastDisplayedLastUpdate;
  bool               firstDisplayedKnown;
  bool               lastDisplayedKnown;
  Time_t             highestAdvancedTime;  // Time of highest time advance
  Time_t             currentTime;
  Time_t             maxHistoryTime;
  Time_t             minHistoryTime;
  // nodeRadius deprecated, and moved to the node object
  //int          nodeRadius; // in pixels
  // Should move this to link also, (not done yet)
  int          linkWidth;  // in pixels
  bool         paused;
  int          framesPerSecond;  // For movie logging
  double       logicalToPixelScale; // Do we need separate X/Y?  Dont think so
  double       pixelToLogicalScale; // Do we need separate X/Y?  Dont think so
  double       zoomScale;           // Zooming scale factor
  bool         endOfData;           // True if no more advances possible
  bool         inUpdate;            // Prevents re-entering the update 
  bool         stalled;             // True if parser is behind desired time

  bool         trackAll;
  bool         PacketOn;
  bool         AnimationStopped;

  void SetShowNodeId();
  void ResetShowNodeId();
  bool IsShowNodeId();
  void increaseNodeIdfontSize();
  void decreaseNodeIdfontSize();
  int getNodeIdFontSize();

  QPointer <QPushButton> quitButton;
  bool         ShowNodeId;
  double           NodeIdFontSize;

  QPointer <QHBoxLayout> hbox1 ;

  void SetAnimatorStatusBar ();
  void ResetAnimatorStatusBar ();


#ifdef WITH_NS3
  void SetOnlineHBox ();
  void ResetOnlineHBox ();
  virtual void focusInEvent( QFocusEvent* );
  void NotifyAnimationOnline (const char *s);
  bool animationOnline;
  void InitAnimOnlineXML ();
  bool animOnlineInitComplete;
  int AnimOnlineThreadIndex;
  AnimOnlineParser * pAnimOnlineParser;
  AnimOnlineDocHandlers * pAnimOnlineDocHandlers;
  int GetAnimOnlineThreadIndex ();
  void NetViewTemporaryHack ();
#endif
  Time_t       advanceToTime;


private:
  double        updateRates[MAX_SLIDER];
  QWidget * centralWidget;
  MainInfo * mainInfo;
public:

  QWidget * Create(QApplication*); // Creates the view windows
  QWidget* mw;                    // main window
public slots:
  void TimeToUpdate();
  void trackTimerSlot();
  void TimeToRecord();
  void WirelessCircles();
  void Play();
  void PlayReverse();
  void FFwdPressed(int);
  void FFwdReleased();
  void Pause();
  void Record();
  void ZoomIn();
  void ZoomOut();
  void ZoomReset();
  void BackupSliderMoved(int);
  void PacketToggle();
  void RateSliderMoved(int);
#ifdef WITH_NS3
  bool IsAnimationStopped();
  void StopAnimationInterfaceSlot();
  void animationOnline_slot(int);
#endif
};

} // namespace netanim
#endif
