/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: George F. Riley<riley@ece.gatech.edu>
 */
// Network Animator Node class

#include <iostream>
#include <math.h>
#include <map>
#include "Node.h"
#include "WiredLink.h"

#include <QPointF>
#include <QRectF>


using namespace std;

namespace netanim {

// Node data/methods
Node::Node()
  : lpId(-1), id(-1), shape(CIRCLE),
    visible(false), exists(false), imageScale(1.0), image(nil), radius(0)
{ // Default constructor is only used when indexing the map of
  // all nodes with an invalid id; this gives a node with exists = false.
}

Node::Node(LpId_t lpId0, NodeId_t id0,
           Meters_t x0, Meters_t y0, Meters_t z0, int r0)
  : lpId(lpId0), id(id0), position(x0, y0, z0), shape(CIRCLE),
    visible(true), exists(true), imageScale(1.0), image(nil), radius(r0)
{
}

Node::Node(LpId_t lpId0, NodeId_t id0, const Point3& p3, int r0)
  : lpId(lpId0), id(id0), position(p3), shape(CIRCLE),
    visible(true), exists(true), imageScale(1.0), image(nil), radius(r0)
{
}

Node::Node(LpId_t lpId0, NodeId_t id0, const Point2& p2, int r0)
  : lpId(lpId0), id(id0), position(p2.x, p2.y, 0), shape(CIRCLE),
    visible(true), exists(true), imageScale(1.0), image(nil), radius(r0)
{
}


void Node::SetShape(NodeShape_t s)
{
  shape = s;
}


void Node::Show()
{
  visible = true;
}

void Node::Hide()
{
  visible = false;
}

WiredLink Node::FindLink(NodeId_t n2Id)
{
  if (exists)
    {
      for (Size_t i = 0; i < links.size(); ++i)
        {
          if (links[i].n2 == n2Id) return links[i];
        }
    }
  // Not found, return empty link
  return WiredLink();
}

void Node::AddLink(const WiredLink& lk)
{
  links.push_back(lk);
}

void Node::AddMobility(const netanim::Mobility& m)
{
  mobility.insert(m);
}

QPointF Node::GetLoc2d(Time_t now) const
{ // Run through the mobility items and update position appropriately
  Point3 currentPosition(position);
  for (netanim::MobilitySet_t::iterator i = mobility.begin(); i != mobility.end(); ++i)
    {
      const netanim::Mobility& m = *i;
      if (m.startTime > now) continue; // Start time in future, ignore
      if (m.endTime < now)
        { // THis one is past, update to end position
          currentPosition = m.endPosition;
        }
      else if (m.startTime < now && m.endTime > now)
        { // Still moving, interpolate
          double frac = (now - m.startTime) / (m.endTime - m.startTime);
          double deltaX = m.endPosition.x - m.startPosition.x;
          double deltaY = m.endPosition.y - m.startPosition.y;
          double x = m.startPosition.x + deltaX * frac;
          double y = m.startPosition.y + deltaY * frac;
          currentPosition = Point3(x, y, 0);
        }
    }
  return QPointF(currentPosition.x, currentPosition.y);
}
} //namespace netanim

