/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: John Abraham <john.abraham@gatech.edu>
 */

#ifdef WITH_NS3
#include "MainInfo/MainInfo.h"
#include "MainWindow/mainwindow.h"
#include "applicationstab.h"
#include "ns3/core-module.h"
#include "ns3/network-module.h"

using namespace ns3;

namespace netanim {

static int defaultGridMaxColumn = 4;
static bool gLogToFile = false;


ApplicationsTab::ApplicationsTab(QObject * parent):AnimTab (parent),
    polling(false),
    simtimePolling(true),
    simtimePollingTimer(ns3::Timer::CANCEL_ON_DESTROY)

{
    wallClocktimer = new QTimer;
}

QWidget* ApplicationsTab::Create(QApplication *app, MainInfo *mi)
{

    (void)app;
    mainInfo = mi;
    numNodes = ns3::NodeList::GetNNodes();

    vbox0 = new QVBoxLayout;
    centralWidget = new QWidget;
    hbox0 = new QHBoxLayout;
    hbox0->addWidget(mi->GetCommonVCR());


    logtoFileButton = new QPushButton("Log to File");
    logtoFileButton->setToolTip("Toggle this button to Log the statistics to the logs folder");
    logtoFileButton->setCheckable(true);
    connect(logtoFileButton, SIGNAL (clicked()), this, SLOT (logtofileslot()));


    QPointer <QLabel> pollLabel = new QLabel ("Poll Interval in ms");
    pollLineEdit = new QLineEdit;
    pollLineEdit->setText("1000");
    pollLabel->setBuddy(pollLineEdit);


    applyButton = new QPushButton(("&Stopped"));
    applyButton->setCheckable(true);
    connect(applyButton, SIGNAL (clicked ()),this,SLOT(apply()));

    polling_gb = new QGroupBox;
    wallClock_rb = new QRadioButton("WallClock");
    wallClock_rb->setToolTip("Use Wall Clock Time for polling");
    connect(wallClock_rb, SIGNAL(clicked(bool)),this, SLOT(wallclockrb_slot(bool)));
    QPointer <QHBoxLayout> pollinggb_hbox = new QHBoxLayout;
    simulationTime_rb = new QRadioButton("SimTime");
    simulationTime_rb->setToolTip("Use Simulation Time for polling");
    simulationTime_rb->setChecked(true);
    connect(simulationTime_rb, SIGNAL (clicked(bool)),this,SLOT(simtimerb_slot(bool)));
    pollinggb_hbox->addWidget(simulationTime_rb);
    pollinggb_hbox->addWidget(wallClock_rb);
    polling_gb->setLayout(pollinggb_hbox);



    hbox0->addWidget(logtoFileButton);
    hbox0->addWidget(pollLabel);
    hbox0->addWidget(pollLineEdit);
    hbox0->addWidget(polling_gb);
    hbox0->addWidget(applyButton);
    vbox0->addLayout(hbox0);

    // Node HBox
    AddApplicationCombos();
    vbox0->addWidget(nodesSA);

    // Gridbox
    gridbox2 = new QGridLayout;
    vbox0->addLayout(gridbox2);

    centralWidget->setLayout(vbox0);
    return centralWidget;


}

void ApplicationsTab::simtimerb_slot(bool state)
{
    if(simulationTime_rb->isChecked())
        simtimePolling = true;
}

void ApplicationsTab::wallclockrb_slot(bool state)
{

    if(wallClock_rb->isChecked())
        simtimePolling = false;
}


void ApplicationsTab::onFocus()
{
    hbox0->insertWidget(0,mainInfo->GetCommonVCR());
    mainInfo->GetCommonVCR()->show();

}

void ApplicationsTab::onLostFocus()
{


}

QWidget* ApplicationsTab::getCentralWidget()
{
    return centralWidget;
}

QString ApplicationsTab::GetTabName()
{

    return "Applications";
}

void ApplicationsTab::logtofileslot()
{
    gLogToFile = !gLogToFile;
    if(gLogToFile)
    logtoFileButton->setText("Logging To file!!");
    else
    logtoFileButton->setText("Log to File");

}

void ApplicationsTab::DoSimTimePolling()
{
    mainInfo->getMainWindow()->SetLCD();
    static bool first = true;
    if (first)
    {
       QMessageBox msg;
       msg.setText("Please ensure you used \"Simulator::Stop (Seconds (Time_t))\" function\nOr your simulation can run endlessly");
       msg.exec();
    }

    if(simtimePollingTimer.IsRunning())
    simtimePollingTimer.Cancel();
    if(!polling || (Simulator::IsFinished() && !first))
        return;
    if(Simulator::IsFinished() && !first)
    {
       polling = false;
       QMessageBox msg;
       msg.setText("Simulation Finished");
       msg.exec();
       applyButton->setText("Stopped");

       return;
    }
    simtimePollingTimer.SetFunction (&netanim::ApplicationsTab::DoSimTimePolling,this);
    UpdateApplications();
    simtimePollingTimer.Schedule(ns3::MilliSeconds(pollString.toInt()));
    first = false;


}


void ApplicationsTab::DoWallClockPolling()
{
    mainInfo->getMainWindow()->SetLCD();
    if(wallClocktimer->isActive())
        wallClocktimer->stop();
    if(!polling)
        return;

    wallClocktimer->setInterval(pollString.toInt());
    connect(wallClocktimer, SIGNAL(timeout()), this, SLOT(UpdateApplications()));
    wallClocktimer->start();


}

void ApplicationsTab::apply()
{
    polling = !polling;
    pollString = pollLineEdit->text();

    if (simtimePolling)
        DoSimTimePolling();
    else
        DoWallClockPolling();


    if (!polling)
    {
        applyButton->setText("Stopped");
        polling_gb->setEnabled(true);
        pollLineEdit->setEnabled(true);
        return;
    }
    else
    {
        applyButton->setText("Polling");
        polling_gb->setEnabled(false);
        pollLineEdit->setEnabled(false);

    }


}


void ApplicationsTab::AddApplicationCombos()
{
    nodesHBox = new QHBoxLayout;
    nodesSA = new QScrollArea;
    for (uint32_t i = 0; i < numNodes ; i++)
    {
        QPointer <ApplicationsCombo> acmb = AddApplicationCombo(i);
        nodesHBox->addWidget(acmb->getButton());
    }
    QWidget * qw = new QWidget;
    qw->setLayout(nodesHBox);
    nodesSA->setFixedHeight(50);
    nodesSA->setWidget(qw);


}


void ApplicationsTab::UpdateApplications()
{

    if(mainInfo->IsSimulationPaused())
        return;
   for (mAType::iterator i = mApplicationsCombo.begin();
    i != mApplicationsCombo.end();
    i++)
   {
       QPointer <ApplicationsCombo> acmb = i->second;
       acmb->UpdateApplications();
   }
}


ApplicationsCombo * ApplicationsTab::AddApplicationCombo(uint32_t nodeId)
{

    ApplicationsCombo * acmb = new ApplicationsCombo(this,nodeId,mainInfo);
    mApplicationsCombo[nodeId] = acmb;
    return acmb;

}


void ApplicationsTab::AdjustGrid()
{
    typedef std::map <uint32_t,QPointer <ApplicationsCombo> > mAType;
    for (mAType::iterator i = mApplicationsCombo.begin();
         i != mApplicationsCombo.end();
         i++)
    {

        QPointer <ApplicationsCombo> p = i->second;
        gridbox2->removeWidget(p->getCentralWidget());
        p->getCentralWidget()->hide();
    }

    for (mAType::iterator i = mApplicationsCombo.begin();
         i != mApplicationsCombo.end();
         i++)
    {
        QPointer <ApplicationsCombo> p = i->second;
        if(p->isactive())
        {
            int total = gridbox2->count();
            int r = gridbox2->rowCount();
            int maxcolumn = defaultGridMaxColumn;
            if ((total % maxcolumn) == 0)
             gridbox2->addWidget(p->getCentralWidget(),r+1,0,1,1);
            else
                gridbox2->addWidget(p->getCentralWidget(),r-1,(total % maxcolumn ),1,1);

            p->getCentralWidget()->show();
        }

    }


}


void ApplicationsTab::NotifyToggled(QPointer <ApplicationsCombo> acmb, uint32_t NodeId,bool active,QWidget * w)
{
    int index=0;
    if ((index=gridbox2->indexOf(w)) == -1)
    {
    // lets add this widget
        if(active)
          {
             int total = gridbox2->count();
             int r = gridbox2->rowCount();
             int maxcolumn = defaultGridMaxColumn;
             if ((total % maxcolumn) == 0)
              gridbox2->addWidget(w,r+1,0,1,1);
             else
              gridbox2->addWidget(w,r-1,(total % maxcolumn ),1,1);

          }
        mApplicationsCombo[NodeId] = acmb;
    }
    if(!active)
    {

         gridbox2->removeWidget(w);
         w->setVisible(false);
         AdjustGrid();

    }
    else
    {
         w->setVisible(true);
    }

}

ApplicationsCombo::ApplicationsCombo(ApplicationsTab *at, uint32_t nodeId,MainInfo * mi):
    mainInfo(mi),
    logtoFile(false),
    NApplications(0)
{
    m_at = at;
    m_nodeId = nodeId;
    //centralWidget = new QWidget;
    nodeButton = new QPushButton(QString::number(nodeId));
    nodeButton->setCheckable(true);
    connect( nodeButton, SIGNAL(clicked()), this, SLOT(onbuttonToggled()) );

    vbox0 = new QVBoxLayout;
    centralWidget = new QWidget;

    gb = new QGroupBox("Node:" + QString::number(nodeId));


    NApplicationsLabel = new QLabel();
    n = ns3::NodeList::GetNode(m_nodeId);
    NS_ASSERT(n);
    NApplications = n->GetNApplications();
    NApplicationsLabel->setText("Num Applications:" + QString::number(NApplications));

    vbox0->addWidget(NApplicationsLabel);
    vbox0->setSpacing(0);

    for (uint32_t i = 0 ; i < NApplications ; i++)
    {
        QPointer <ApplicationCombo2> acmb2 = new ApplicationCombo2(n,n->GetApplication(i));
        vApplicationCombos.push_back(acmb2);
        vbox0->addWidget(acmb2->getCentralWidget());
    }

    gb->setLayout(vbox0);

    centralWidget = gb;
    active = false;

}

ApplicationsCombo::~ApplicationsCombo()
{

}



void ApplicationsCombo::SetLogtoFile(bool logit)
{
    logtoFile = logit;
}


bool ApplicationsCombo::isactive()
{
    return active;
}

QPointer <QPushButton> ApplicationsCombo::getButton()
{
    return nodeButton;

}


void ApplicationsCombo::onbuttonToggled()
{
    active = !active;
    m_at->NotifyToggled(QPointer <ApplicationsCombo> (this),m_nodeId, active,centralWidget);

}


QWidget * ApplicationsCombo::getCentralWidget()
{
    return centralWidget;
}


void ApplicationsCombo::UpdateApplications()
{

    for (uint32_t i = 0 ; i < vApplicationCombos.size() ; i++)
    {
        vApplicationCombos[i]->UpdateApplication();
    }
}





ApplicationCombo2::ApplicationCombo2(ns3::Ptr<ns3::Node> node, ns3::Ptr<ns3::Application> pApp):
    m_node(node)
{
    centralWidget = new QWidget;
    vbox0 = new QVBoxLayout;
    sAppName = pApp->GetInstanceTypeId().GetName();
    m_app = pApp;
    appName = new QLabel;
    QString qAppName = sAppName.c_str();
    appName->setText("Type:" + qAppName.split("::")[1]);
    QFont f;
    f.setUnderline(true);
    appName->setFont(f);
    vbox0->addWidget(appName);
    vbox0->setSpacing(0);
    centralWidget->setLayout(vbox0);
    vCounters.push_back(new QLabel);

    vbox0->addWidget(vCounters[0]);

    ResetStats();
    ConnectTraces();


}

void ApplicationCombo2::UpdateApplication()
{
    std::string InstanceName = m_app->GetInstanceTypeId().GetName();
    //std::cout<<"InstanceName:"<<InstanceName<<std::endl;
    // PacketSink
    if (InstanceName == "ns3::PacketSink")
        UpdatePacketSinkWidget();

    // BulkSend
    else if (InstanceName == "ns3::BulkSendApplication")
        UpdateBulkSendWidget();

    // OnOff
    else if (InstanceName == "ns3::OnOffApplication")
        UpdateOnOffWidget();

    // UDPEchoClient

    else if (InstanceName == "ns3::UdpEchoClient")
        UpdateUdpEchoClientWidget();

    // v4Ping
    else if (InstanceName == "ns3::V4Ping")
        Updatev4PingWidget();


}


void ApplicationCombo2::UpdatePacketSinkWidget()
{
    QPointer <QLabel> l = vCounters[0];

    for (mInetSocketAddressCounters_t::iterator i = mInetSocketAddress_Plots.begin();
         i != mInetSocketAddress_Plots.end();
          i++)
    {

        ApplicationCombo3 * pacmb3 = i->second;
        pacmb3->timeToPlot();
        ns3::InetSocketAddress inetaddr = i->first;
        uint32_t rxBytesCounter = mInetSocketAddress[inetaddr];
        mInetSocketAddressTput_Plots[inetaddr]->setStat(rxBytesCounter/Simulator::Now().GetSeconds());
        mInetSocketAddressTput_Plots[inetaddr]->timeToPlot();
    }

}
void ApplicationCombo2::PacketSink_Rx_cb( ns3::Ptr<const ns3::Packet> p, const Address & addr)
{

    ns3::InetSocketAddress inetaddress = ns3::InetSocketAddress::ConvertFrom(addr);
    ns3::Ipv4Address ipv4addr = inetaddress.GetIpv4();
    uint16_t port = inetaddress.GetPort();

    mInetSocketAddress[inetaddress] += p->GetSize();


    if (mInetSocketAddress_Plots.find(inetaddress) == mInetSocketAddress_Plots.end())
    {
        std::stringstream ss;
        ipv4addr.Print(ss);
        ss << ":" << port;
        QString ipstring = ss.str().c_str();
        mInetSocketAddress_Plots[inetaddress] = new ApplicationCombo3("PacketSink","Bytes Rcvd",ipstring);
        vbox0->addWidget(mInetSocketAddress_Plots[inetaddress]->getCentralWidget());

        mInetSocketAddressTput_Plots[inetaddress] = new ApplicationCombo3("PacketSink","Thrput Bps",ipstring);
        vbox0->addWidget(mInetSocketAddressTput_Plots[inetaddress]->getCentralWidget());


    }
    ApplicationCombo3 * pacmb3 = mInetSocketAddress_Plots[inetaddress];
    pacmb3->setStat(mInetSocketAddress[inetaddress]);
}

void ApplicationCombo2::ConnectPacketSinkTraces()
{
    ns3::Ptr <ns3::PacketSink> ps = DynamicCast<ns3::PacketSink> (m_app);
    NS_ASSERT(ps);
    ps->TraceConnectWithoutContext("Rx",MakeCallback(&ApplicationCombo2::PacketSink_Rx_cb,this));

}

void ApplicationCombo2::UpdateBulkSendWidget()
{
    for (mTraceName_t::iterator i = mTraceNameCounters.begin();
         i != mTraceNameCounters.end();
          i++)
    {

        ApplicationCombo3 * pacmb3 = i->second;
        pacmb3->timeToPlot();
    }

}

void ApplicationCombo2::BulkSend_Tx_cb( ns3::Ptr<const ns3::Packet> p)
{
    BulkSend_Tx_stat++;
    if (mTraceNameCounters.find("Tx") == mTraceNameCounters.end())
    {

        mTraceNameCounters["Tx"] = new ApplicationCombo3("BulkSend","Tx","NodeId:"+ QString::number(m_node->GetId()));
        vbox0->addWidget(mTraceNameCounters["Tx"]->getCentralWidget());
    }
    mTraceNameCounters["Tx"]->incStat();


}

void ApplicationCombo2::ConnectBulkSendTraces()
{
    ns3::Ptr <ns3::BulkSendApplication> bs = DynamicCast<ns3::BulkSendApplication> (m_app);
    bs->TraceConnectWithoutContext("Tx",MakeCallback(&ApplicationCombo2::BulkSend_Tx_cb,this));
}


void ApplicationCombo2::UpdateOnOffWidget()
{
    for (mTraceName_t::iterator i = mTraceNameCounters.begin();
         i != mTraceNameCounters.end();
          i++)
    {

        ApplicationCombo3 * pacmb3 = i->second;
        pacmb3->timeToPlot();
    }


}

void ApplicationCombo2::OnOff_Tx_cb( ns3::Ptr< const ns3::Packet> p)
{
    OnOff_Tx_stat++;
    if (mTraceNameCounters.find("Tx") == mTraceNameCounters.end())
    {

        mTraceNameCounters["Tx"] = new ApplicationCombo3("OnOff","Tx","NodeId:"+ QString::number(m_node->GetId()));
        vbox0->addWidget(mTraceNameCounters["Tx"]->getCentralWidget());
    }
    mTraceNameCounters["Tx"]->incStat();


}

void ApplicationCombo2::ConnectOnOffTraces()
{
    ns3::Ptr <ns3::OnOffApplication> of = DynamicCast<ns3::OnOffApplication> (m_app);
    of->TraceConnectWithoutContext("Tx",MakeCallback(&ApplicationCombo2::OnOff_Tx_cb,this));


}

void ApplicationCombo2::UpdateUdpEchoClientWidget()
{
    for (mTraceName_t::iterator i = mTraceNameCounters.begin();
         i != mTraceNameCounters.end();
          i++)
    {

        ApplicationCombo3 * pacmb3 = i->second;
        pacmb3->timeToPlot();
    }

}
void ApplicationCombo2::UdpEchoClient_Tx_cb( ns3::Ptr<const ns3::Packet> p)
{
    UdpEchoClient_stat++;
    if (mTraceNameCounters.find("Tx") == mTraceNameCounters.end())
    {

        mTraceNameCounters["Tx"] = new ApplicationCombo3("UdpEchoClient","Tx","NodeId:"+ QString::number(m_node->GetId()));
        vbox0->addWidget(mTraceNameCounters["Tx"]->getCentralWidget());
    }
    mTraceNameCounters["Tx"]->incStat();
}

void ApplicationCombo2::ConnectUdpEchoClientTraces()
{
    ns3::Ptr <ns3::UdpEchoClient> ue = DynamicCast<ns3::UdpEchoClient> (m_app);
    ue->TraceConnectWithoutContext("Tx",MakeCallback(&ApplicationCombo2::OnOff_Tx_cb,this));

}


void ApplicationCombo2::Updatev4PingWidget()
{
    for (mTraceName_t::iterator i = mTraceNameCounters.begin();
         i != mTraceNameCounters.end();
          i++)
    {

        ApplicationCombo3 * pacmb3 = i->second;
        pacmb3->timeToPlot();
    }

}

void ApplicationCombo2::v4Ping_Rtt_cb(ns3::Time t)
{
   v4Ping_Rtt_stat = t;

   if (mTraceNameCounters.find("Rtt") == mTraceNameCounters.end())
   {

       mTraceNameCounters["Rtt"] = new ApplicationCombo3("v4Ping","Rtt","NodeId:"+ QString::number(m_node->GetId()));
       vbox0->addWidget(mTraceNameCounters["Rtt"]->getCentralWidget());
   }
   mTraceNameCounters["Rtt"]->setStat(t.ToDouble(ns3::Time::S));

}

void ApplicationCombo2::Connectv4PingTraces()
{
    ns3::Ptr <ns3::V4Ping> v4 = DynamicCast<ns3::V4Ping> (m_app);
    v4->TraceConnectWithoutContext("Rtt",MakeCallback(&ApplicationCombo2::v4Ping_Rtt_cb,this));


}

void ApplicationCombo2::ConnectTraces()
{

    std::string InstanceName = m_app->GetInstanceTypeId().GetName();
    //std::cout<<"InstanceName:"<<InstanceName<<std::endl;
    // PacketSink
    if (InstanceName == "ns3::PacketSink")
        ConnectPacketSinkTraces();

    // BulkSend
    else if (InstanceName == "ns3::BulkSendApplication")
        ConnectBulkSendTraces();

    // OnOff
    else if (InstanceName == "ns3::OnOffApplication")
        ConnectOnOffTraces();

    // UDPEchoClient

    else if (InstanceName == "ns3::UdpEchoClient")
        ConnectUdpEchoClientTraces();

    // v4Ping
    else if (InstanceName == "ns3::V4Ping")
        Connectv4PingTraces();
}

void ApplicationCombo2::ResetStats()
{
    Reset_PacketSinkStats();
    Reset_BulkSendStats();
    Reset_OnOffStats();
    Reset_UDPEchoClientStats();
    Reset_v4PingStats();
}

void ApplicationCombo2::Reset_PacketSinkStats()
{
    PacketSink_Rx_stat = 0;

}

void ApplicationCombo2::Reset_BulkSendStats()
{
    BulkSend_Tx_stat = 0;

}

void ApplicationCombo2::Reset_OnOffStats()
{

    OnOff_Tx_stat = 0;
}


void ApplicationCombo2::Reset_UDPEchoClientStats()
{
    UdpEchoClient_stat = 0;

}

void ApplicationCombo2::Reset_v4PingStats()
{
    v4Ping_Rtt_stat = ns3::Time(0);
}


QWidget * ApplicationCombo2::getCentralWidget()
{
    return centralWidget;

}


ApplicationCombo3::ApplicationCombo3(QString appName, QString traceName, QString auxString):
    pgrapher(0),
    theCounter(0),
    plotOn(false),
    m_appName(appName),
    m_traceName(traceName),
    plotCounter(0),
    m_auxString(auxString)

{
    file = new QFile("logs/" + appName.replace(" ","_") + "_" + auxString.replace(":","_") + "_" + traceName + ".txt");
    pCounterLabel = new QLabel;
    centralWidget = new QWidget;
    hbox0 = new QHBoxLayout;
    hbox0->addWidget(pCounterLabel);
    pCounterLabel->setText(m_auxString +":" + m_traceName + ":" + QString::number(theCounter) );
    plotButton = new QPushButton ("Plot");
    connect(plotButton, SIGNAL (clicked()),this,SLOT(plotSlot()));
    hbox0->addWidget(plotButton);
    centralWidget->setLayout(hbox0);
}



void ApplicationCombo3::OpenFile(QPointer <QFile> f)
{
    if (f->isOpen())
        return;
    if (!f->open(QIODevice::WriteOnly))
    {
        std::cout<<"Unable to open:"<<f->fileName().toAscii().data()<<endl;
        exit(1);
    }


}
void ApplicationCombo3::AppendToFile(QPointer <QFile> f,QString str)
{
    OpenFile (f);

    if(f->write(str.toAscii().data()) == -1)
    {

        std::cout<<"Write Failed:"<<endl;
        exit(1);
    }
    f->flush();
}



QWidget * ApplicationCombo3::getCentralWidget()
{
    return centralWidget;
}


TwoDGrapherView* ApplicationCombo3::getTwodGrapher()
{

    return pgrapher;
}

void ApplicationCombo3::incStat()
{
    theCounter++;
    plotCounter++;
}


void ApplicationCombo3::setStat(double t)
{
    theCounter = t;
    plotCounter = t;
}

void ApplicationCombo3::timeToPlot()
{
    if(pgrapher)
    pgrapher->AppendDataPoint(Simulator::Now().GetSeconds(),plotCounter);
    pCounterLabel->setText(m_auxString +":" + m_traceName + ":" + QString::number(theCounter) );
    if(gLogToFile)
        AppendToFile(file,QString::number(Simulator::Now().GetSeconds()) +"\t" + QString::number(plotCounter) + "\n");
}

 void ApplicationCombo3::plotSlot()
{
    plotOn = !plotOn;
    if(plotOn)
    {
        pgrapher = new TwoDGrapherView(m_appName+":" + m_auxString,"Simulation Time",m_traceName,m_appName+":"+m_auxString);//,this);
        hbox0->addWidget(pgrapher);
        plotButton->setText("Plot Off");
        pgrapher->setMinimumSize(500,500);
    }
    else
    {
        plotButton->setText("Plot");
        delete pgrapher;
        pgrapher = 0;
    }



}


} // namespace netanim

#endif //WITH_NS3
