#ifdef WITH_NS3
/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: John Abraham <john.abraham@gatech.edu>
 */


#include "Graphing/TwodGrapher.h"
#include <iostream>


namespace netanim {


TwoDGrapherView::TwoDGrapherView(
                                 QString charTitle,
                                 QString XaxisLabel,
                                 QString YaxisLabel,
                                 QString snapShotBaseName,
                                 QWidget *parent) :
    QGraphicsView(parent),
    xAxisMargin(100),
    yAxisMargin(100),
    xAxisDataSections(10),
    yAxisDataSections(10),
    nDataPoints(100),
    m_snapShotBaseName(snapShotBaseName),
    gridLinesOn(true),
    showOptions(true)
{

    scene = new QGraphicsScene;

    datagenTimer = new QTimer;
    datagenTimer->setInterval(1000);
    connect(datagenTimer, SIGNAL(timeout()),this,SLOT(onTimeout()));
    //datagenTimer->start();

    setScene(scene);
    setSceneRect(0,0,500,500);
    setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    InitOptions();
    setXAxisLabel(XaxisLabel);
    setYAxisLabel(YaxisLabel);
    InitAxisLabels();
    setAlignment(Qt::AlignLeft);

    AddAxes();
    InitXAxisPoints();
    InitYAxisPoints();
    InitPlotPoints();

    charTitleLabel = new QLabel;
    chartTitlegWidget = scene->addSimpleText(charTitle);
    chartTitlegWidget->setPos(sceneRect().width()/4,yAxisMargin/2);

    show();


}


void TwoDGrapherView::setChartTitle(QString title)
{
    charTitleLabel->setText(title);
}

void TwoDGrapherView::InitOptions()
{


    optionsWidget = new QGroupBox("Options");

    QPointer <QHBoxLayout> hbox = new QHBoxLayout;
    grid_cb = new QCheckBox;
    grid_cb->setChecked(true);
    grid_cb->setText("Grid-lines");
    connect(grid_cb,SIGNAL (stateChanged(int)),this,SLOT(grid_cb_slot(int)));
    hbox->addWidget(grid_cb);
    optionsWidget->setLayout(hbox);
    dataPointsLabel = new QLabel ("DataPoints:"+ QString::number(nDataPoints));
    hbox->addWidget(dataPointsLabel);
    plusDP = new QPushButton("+");
    connect(plusDP,SIGNAL (clicked()),this,SLOT(increaseDataPoints_slot()));
    minusDP = new QPushButton("-");
    connect(minusDP,SIGNAL (clicked()),this,SLOT(decreaseDataPoints_slot()));
    hbox->addWidget(plusDP);
    hbox->addWidget(minusDP);

    QPointer <QPushButton> snapButton = new QPushButton("Snap");
    snapButton->setToolTip("Take a snapshot of the graph.Store it in the logs folder");
    hbox->addWidget(snapButton);
    connect(snapButton,SIGNAL(clicked()),this,SLOT(snapshot_slot()));

    QPointer <QPushButton> showallButton = new QPushButton("Show All");
    showallButton->setToolTip("Show all data points");
    hbox->addWidget(showallButton);
    connect(showallButton,SIGNAL(clicked()),this,SLOT(showall_slot()));

    scene->addWidget(optionsWidget)->setPos(0,0);
    optionsWidget->hide();

}

void TwoDGrapherView::showall_slot()
{

    while (nDataPoints < mDataPoints.size())
    {
        increaseDataPoints();
    }
    setXAxisPoints();
    setYAxisPoints();
    SetPlotPoints();


}
void TwoDGrapherView::snapshot_slot()
{
    QPixmap pixmap = QPixmap::grabWidget(this);
    QString curTime = QTime::currentTime().toString().replace("/","_");
    QString fileLocation = "logs/" + m_snapShotBaseName +"_" + curTime +".png";
    pixmap.save(fileLocation);
    std::cout<<"Saved to "<< fileLocation.toAscii().data()<< std::endl;


}

void TwoDGrapherView::options_gb_slot()
{
    showOptions = !showOptions;
    if(showOptions)
    {
        showOptionsButton->setText("Hide options");
        optionsWidget->setVisible(true);

    }
    else
    {
        showOptionsButton->setText("Show options");
        optionsWidget->setVisible(false);

    }

}

void TwoDGrapherView::increaseDataPoints_slot()
{
    increaseDataPoints();
    dataPointsLabel->setText("DataPoints:"+ QString::number(nDataPoints));

}

void TwoDGrapherView::decreaseDataPoints_slot()
{
    decreaseDataPoints();
    dataPointsLabel->setText("DataPoints:"+ QString::number(nDataPoints));

}
void TwoDGrapherView::grid_cb_slot(int state)
{

    if(grid_cb->isChecked())
        gridLinesOn = true;
    else
        gridLinesOn = false;

    for (uint32_t i = 0; i < vXGridLines.size() ; i++)
        {
            if(gridLinesOn)
                vXGridLines[i]->show();
            else
                vXGridLines[i]->hide();
        }
    for (uint32_t i = 0; i < vYGridLines.size() ; i++)
        {
            if(gridLinesOn)
                vYGridLines[i]->show();
            else
                vYGridLines[i]->hide();
        }



}

void TwoDGrapherView::setGridLines(bool s)
{
    gridLinesOn = s;

}

void TwoDGrapherView::InitPlotPoints()
{
    for (uint32_t i =  0 ; i < nDataPoints ; i++ )
    {
        QLine l;
        vPlotPoints.push_back(scene->addLine(l));
    }
}


void TwoDGrapherView::SetPlotPoints()
{
    uint32_t count = nDataPoints;
    uint32_t j = 0;
    if (mDataPoints.empty())
        return;
    double xmax,xmin;
    getXDataMinMax(&xmin,&xmax);

    double ymin,ymax;
    getYDataMinMax(&ymin,&ymax);


    for (DataPointMap_t::reverse_iterator i = mDataPoints.rbegin();
         ((i != mDataPoints.rend())  && count);
         i++,count--,j++)
    {
        QGraphicsLineItem * li = vPlotPoints[j];
        double x = i->first;
        double y = i->second;


        double translatedX =  xAxisStart.x() + ((pXAxisLine->line().length()/(xmax-xmin)) * (x-xmin));
        // std::cout<<"xmax-xmin:"<<(xmax-xmin)<<std::endl;



        double translatedY =  xAxisStart.y() - (((pYAxisLine->line().length() )/(ymax-ymin)) * y);
      //  std::cout<<"Length:" << pYAxisLine->line().length() << " Ymax:"<<ymax<<" y:"<<y<<std::endl;


        QLineF line(translatedX,xAxisStart.y(), translatedX,translatedY);
        QPen p(Qt::red);
        li->setLine(line);
        li->setPen(p);

    }
   // std::cout<<"nDataPoints:"<<nDataPoints<<" j:"<<j<<std::endl;

}




void TwoDGrapherView::resizeEvent(QResizeEvent *event)
{
    QGraphicsView::resizeEvent(event);
    setSceneRect(rect());
    QPixmap pm(viewport()->size());
    QPainter p(&pm);
    drawAxisLabels(&p,rect());
    drawAxis(&p,rect());
    setXAxisPoints();
    setYAxisPoints();
    SetPlotPoints();
    chartTitlegWidget->setPos(sceneRect().width()/4,yAxisMargin/2);
}



void TwoDGrapherView::onTimeout()
{
    static int x = 0;

    double y = rand() % 1000;
    //std::cout<<"Rand Y:"<<y<<std::endl;
    AppendDataPoint(x,y);
    x++;
    datagenTimer->start();
}


void TwoDGrapherView::setAxisLabelPosition()
{

}

void TwoDGrapherView::paintEvent(QPaintEvent *event)
{
   setSceneRect(rect());
   QGraphicsView::paintEvent(event);

}

void TwoDGrapherView::mouseMoveEvent(QMouseEvent *event)
{

    QPoint p = event->pos();
    if((p.x() >=0 && p.x() <= rect().right()) &&
        (p.y() >=0 && p.y() <= yAxisMargin))
    {
        optionsWidget->show();
        chartTitlegWidget->hide();
    }
    else
    {
        optionsWidget->hide();
        chartTitlegWidget->show();
    }
}
void TwoDGrapherView::drawBackground(QPainter *painter, const QRectF &rectbg)
{
    QGraphicsView::drawBackground(painter,rectbg);
}

void PrintRect(QString msg,QRectF r)
{
    std::cout<<msg.toAscii().data() << std::endl;
    std::cout<<" Right,Left"<< r.right() <<","<<r.left()<<std::endl;
    std::cout<<" Top,Bottom"<< r.top() << "," << r.bottom() <<std::endl;
}


void TwoDGrapherView::AddAxes()
{

    // Draw X Axis

    pXAxisLine = scene->addLine(GetXAxisLine());


    // Draw Y Axis

    pYAxisLine = scene->addLine(GetYAxisLine());
    //std::cout<<"X Axis:"<<yAxisStart.x() <<"," << yAxisStart.y() <<std::endl;
    //std::cout<<"Y Axis:"<<yAxisEnd.x() <<"," << yAxisEnd.y() <<std::endl;





}
void TwoDGrapherView::drawAxis(QPainter *painter, const QRectF &rectbg)
{
    // Draw X Axis

    pXAxisLine->setLine(GetXAxisLine());


    // Draw Y Axis

    pYAxisLine->setLine(GetYAxisLine());
    //std::cout<<"X Axis:"<<yAxisStart.x() <<"," << yAxisStart.y() <<std::endl;
    //std::cout<<"Y Axis:"<<yAxisEnd.x() <<"," << yAxisEnd.y() <<std::endl;




}
void TwoDGrapherView::CalcXAxisCoord()
{
    xAxisStart.setX(sceneRect().left() + xAxisMargin);
    xAxisStart.setY(sceneRect().bottom()-xAxisMargin);

    xAxisEnd.setX(sceneRect().right()  - xAxisMargin);
    xAxisEnd.setY(sceneRect().bottom() - xAxisMargin);
    //std::cout<<"X Axis:"<<xAxisStart.x() <<"," << xAxisStart.y() <<std::endl;
    //std::cout<<"Y Axis:"<<xAxisEnd.x() <<"," << xAxisEnd.y() <<std::endl;

}

void TwoDGrapherView::CalcYAxisCoord()
{
    yAxisStart.setX(sceneRect().left() + xAxisMargin);
    yAxisStart.setY(sceneRect().bottom()-xAxisMargin);

    yAxisEnd.setX(sceneRect().left()  + xAxisMargin);
    yAxisEnd.setY(sceneRect().top() + yAxisMargin);




}


QLine TwoDGrapherView::GetXAxisLine()
{
    CalcXAxisCoord();
    QLine l(xAxisStart.x(),xAxisStart.y(),xAxisEnd.x(),xAxisEnd.y());
    return l;
}

QLine TwoDGrapherView::GetYAxisLine()
{

    CalcYAxisCoord();
    QLine l(yAxisStart.x(),yAxisStart.y(),yAxisEnd.x(),yAxisEnd.y());
    return l;

}

void TwoDGrapherView::InitXAxisPoints()
{
    CalcXAxisCoord();
    double inter =  pXAxisLine->line().length()/xAxisDataSections;
    //std::cout<<"inter:"<<inter<<std::endl;
    double xmin,xmax;
    getXDataMinMax(&xmin,&xmax);
    double intervalue = (xmax-xmin)/xAxisDataSections;

    for(uint32_t i = 0 ; i < xAxisDataSections;i++)
    {
        vXAxisPoints.push_back(scene->addEllipse(xAxisStart.x()+(i*inter),xAxisStart.y(),3,3));
        QGraphicsSimpleTextItem * psit = scene->addSimpleText(QString::number(i*intervalue));
        psit->setRotation(90);
        vXAxisPointsText.push_back(psit);
        psit->setPos(xAxisStart.x()+(i*inter),xAxisStart.y());
        QLineF gl;
        vXGridLines.push_back( scene->addLine(gl) );

    }

}


void TwoDGrapherView::InitYAxisPoints()
{
    CalcYAxisCoord();
    double inter =  pYAxisLine->line().length()/yAxisDataSections;
    //std::cout<<"inter:"<<inter<<std::endl;
    double ymin,ymax;
    getYDataMinMax(&ymin,&ymax);
    double intervalue = (ymax-ymin)/yAxisDataSections;

    for(uint32_t i = 0 ; i < yAxisDataSections;i++)
    {
        vYAxisPoints.push_back(scene->addEllipse(yAxisStart.x(),yAxisStart.y() - (i*inter),3,3));
        QGraphicsSimpleTextItem * psit = scene->addSimpleText(QString::number(i*intervalue));
        vYAxisPointsText.push_back(psit);
        psit->setPos(yAxisStart.x(),yAxisStart.y() - (i*inter));
        QLineF gl;
        vYGridLines.push_back( scene->addLine(gl) );

    }


}

void TwoDGrapherView::setYAxisPoints()
{
    CalcYAxisCoord();
    double inter =  pYAxisLine->line().length()/yAxisDataSections;
    //std::cout<<"inter:"<<inter<<std::endl;
    double ymin,ymax;
    getYDataMinMax(&ymin,&ymax);
    double intervalue = (ymax-ymin)/yAxisDataSections;

    for(uint32_t i = 0 ; i < yAxisDataSections;i++)
    {
        QGraphicsEllipseItem * pgi = vYAxisPoints[i];
        pgi->setRect(yAxisStart.x(),yAxisStart.y() - (i*inter),3,3);
        QGraphicsSimpleTextItem * psit = vYAxisPointsText[i];
        psit->setPos(yAxisStart.x()-50,yAxisStart.y() - (i*inter));
        psit->setText(QString::number(i*intervalue));

        if (gridLinesOn)
        {
            QLineF gridLine(xAxisStart.x(),
                            yAxisStart.y() - (i*inter),
                            xAxisEnd.x(),
                            yAxisStart.y() - (i*inter));
            QPen pn(Qt::cyan);

            QGraphicsLineItem * pgl = vYGridLines[i];
            pgl->setLine(gridLine);
            pgl->setPen(pn);

        }

    }


}

void TwoDGrapherView::setXAxisPoints()
{

    if (vXAxisPoints.empty())
        return;
    CalcXAxisCoord();

    double inter =  pXAxisLine->line().length()/xAxisDataSections;
  //  std::cout<<"inter:"<<inter<<std::endl;
    double xmin,xmax;
    getXDataMinMax(&xmin,&xmax);
    double intervalue = (xmax-xmin)/xAxisDataSections;
    //intervalue = std::min(0.0,intervalue);


    for(uint32_t i = 0 ; i < xAxisDataSections ;i++)
    {
        QGraphicsEllipseItem * pgi = vXAxisPoints[i];
      //  std::cout<<"New Pos:"<<xAxisStart.x() +(i*inter) <<","<<xAxisStart.y()<<std::endl;
        pgi->setRect(xAxisStart.x()+(i*inter),xAxisStart.y(),3,3);
        QGraphicsSimpleTextItem * psit = vXAxisPointsText[i];
        psit->setPos(xAxisStart.x()+(i*inter),xAxisStart.y());
       // std::cout<<"Intervalue :"<<intervalue <<" xmin:"<<xmin<<std::endl;
        psit->setText(QString::number((i+xmin)*intervalue));

        if (gridLinesOn)
        {
            QLineF gridLine(xAxisStart.x()+(i*inter),
                            xAxisStart.y(),
                            xAxisStart.x()+(i*inter),
                            yAxisEnd.y());
            QPen pn(Qt::cyan);

            QGraphicsLineItem * pgl = vXGridLines[i];
            pgl->setLine(gridLine);
            pgl->setPen(pn);

        }


    }

}

void TwoDGrapherView::drawAxisLabels(QPainter *painter, const QRectF &rectbg)
{
    setSceneRect(rect());
    XAxisLabel->setPos(sceneRect().width()/2,sceneRect().bottom()-20);
    YAxisLabel->setPos(sceneRect().left() +20,sceneRect().height()/2);

 //   std::cout<<"Xasix:"<<XAxisLabel->text().toAscii().data()<<std::endl;

}

void TwoDGrapherView::InitAxisLabels()
{
    XAxisLabel = scene->addSimpleText(xAxisLabelString);
    XAxisLabel->setPos(rect().width()/2,rect().height()/2);
  //  std::cout<<"XAxi:"<<xAxisLabelString.toAscii().data()<<std::endl;

    YAxisLabel = scene->addSimpleText(yAxisLabelString);
  //  std::cout<<"YAxi:"<<yAxisLabelString.toAscii().data()<<std::endl;

    YAxisLabel->setRotation(90);
    YAxisLabel->setPos(rect().left(),rect().bottom()/2);

}

void TwoDGrapherView::AppendDataPoint(double x, double y)
{

    mDataPoints[x] = y;
    setXAxisPoints();
    setYAxisPoints();
    SetPlotPoints();

}


void TwoDGrapherView::setXAxisLabel(QString l)
{
    xAxisLabelString = l;
}

void TwoDGrapherView::setYAxisLabel(QString l)
{
    yAxisLabelString = l;
}


void TwoDGrapherView::getXDataMinMax(double *x, double *y)
{
    if (mDataPoints.empty())
    {
        *x = 0; *y = 0;
        return;
    }

    *x = 0;
    *y = 0;
    uint32_t count = nDataPoints;
    *y =  (mDataPoints.rbegin())->first;

    //std::cout<<"X Axis:"<<*x<<","<<*y<<std::endl;
    DataPointMap_t::reverse_iterator i = mDataPoints.rbegin();
    for (;
         (i != mDataPoints.rend()) && count;
         i++,count--)

    {
        *x = i->first;

    }

    if (*x == *y)
        *x = 0;
}


void TwoDGrapherView::increaseDataPoints()
{

    nDataPoints++;
    QLine l;
    QPen p;
    p.setWidth(2);
    vPlotPoints.push_back(scene->addLine(l,p));
}

void TwoDGrapherView::decreaseDataPoints()
{
    if(nDataPoints==1)
        return;
    nDataPoints--;
    vPlotPoints.pop_back();

}

void TwoDGrapherView::getYDataMinMax(double *x, double *y)
{
    if (mDataPoints.empty())
    {
        *x = 0; *y = 0;
        return;
    }
    *x = 0;
    *y = 0;
    uint32_t count = nDataPoints;
    for (DataPointMap_t::reverse_iterator i = mDataPoints.rbegin();
         (i != mDataPoints.rend()) && count;
         i++,count--)

    {

        *y = std::max(i->second,*y);
    }
    if (*x == *y)
        *x = 0;


}

} // namespace netanim

#endif // WITH_NS3
