/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2005,2006 INRIA
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Adapted from "Default-simulator-impl.h" in ns-3
 */

#ifdef WITH_NS3

#include "ns3/simulator.h"
#include "animsimulator-impl.h"
#include "Animator/NetView.h"
#include "ns3/scheduler.h"
#include "ns3/event-impl.h"

#include "ns3/ptr.h"
#include "ns3/pointer.h"
#include "ns3/assert.h"
#include "ns3/log.h"

#include <math.h>
#include <sys/time.h>

NS_LOG_COMPONENT_DEFINE ("AnimSimulatorImpl");

namespace ns3 {

NS_OBJECT_ENSURE_REGISTERED (AnimSimulatorImpl);

TypeId
AnimSimulatorImpl::GetTypeId (void)
{
  static TypeId tid = TypeId ("ns3::AnimSimulatorImpl")
    .SetParent<SimulatorImpl> ()
    .AddConstructor<AnimSimulatorImpl> ()
  ;
  return tid;
}

AnimSimulatorImpl::AnimSimulatorImpl ()
{
}

AnimSimulatorImpl::AnimSimulatorImpl (netanim::MainInfo *mi): mainInfo(mi)
{
  m_stop = false;
  // uids are allocated from 4.
  // uid 0 is "invalid" events
  // uid 1 is "now" events
  // uid 2 is "destroy" events
  m_uid = 4;
  // before ::Run is entered, the m_currentUid will be zero
  m_currentUid = 0;
  m_currentTs = 0;
  m_currentContext = 0xffffffff;
  m_unscheduledEvents = 0;
}

AnimSimulatorImpl::~AnimSimulatorImpl ()
{
}

void
AnimSimulatorImpl::DoDispose (void)
{
  while (!m_events->IsEmpty ())
    {
      Scheduler::Event next = m_events->RemoveNext ();
      next.impl->Unref ();
    }
  m_events = 0;
  SimulatorImpl::DoDispose ();
}
void
AnimSimulatorImpl::Destroy ()
{
    if (!mainInfo->GetSimulation()->IsConfigureSimulationComplete())
    {


        QString t = "Simulation::Destroy called before ConfigureSimulation Completed\n \
                     Please remove any Simulation::Destroy statements\n \
                     from Simulation::ConfigureSimulation";
        std::cout << t.toAscii().data() <<std::endl;

        exit(-1);
    }

  while (!m_destroyEvents.empty ())
    {
      Ptr<EventImpl> ev = m_destroyEvents.front ().PeekEventImpl ();
      m_destroyEvents.pop_front ();
      NS_LOG_LOGIC ("handle destroy " << ev);
      if (!ev->IsCancelled ())
        {
          ev->Invoke ();
        }
    }
}

void
AnimSimulatorImpl::SetScheduler (ObjectFactory schedulerFactory)
{
  Ptr<Scheduler> scheduler = schedulerFactory.Create<Scheduler> ();

  if (m_events != 0)
    {
      while (!m_events->IsEmpty ())
        {
          Scheduler::Event next = m_events->RemoveNext ();
          scheduler->Insert (next);
        }
    }
  m_events = scheduler;
}

// System ID for non-distributed simulation is always zero
uint32_t
AnimSimulatorImpl::GetSystemId (void) const
{
  return 0;
}

void
AnimSimulatorImpl::ProcessOneEvent (void)
{
  Scheduler::Event next = m_events->RemoveNext ();

  NS_ASSERT (next.key.m_ts >= m_currentTs);
  m_unscheduledEvents--;

  NS_LOG_LOGIC ("handle " << next.key.m_ts);
  m_currentTs = next.key.m_ts;
  m_currentContext = next.key.m_context;
  m_currentUid = next.key.m_uid;
  next.impl->Invoke ();
  next.impl->Unref ();
}

bool
AnimSimulatorImpl::IsFinished (void) const
{
  return m_events->IsEmpty () || m_stop;
}

uint64_t
AnimSimulatorImpl::NextTs (void) const
{
  NS_ASSERT (!m_events->IsEmpty ());
  Scheduler::Event ev = m_events->PeekNext ();
  return ev.key.m_ts;
}

Time
AnimSimulatorImpl::Next (void) const
{
  return TimeStep (NextTs ());
}


struct timeval startClock;
long timevaldiff(const struct timeval *starttime, const struct timeval *finishtime)
{

  long usec;
  usec=(finishtime->tv_sec-starttime->tv_sec)*1000000;
  usec+=(finishtime->tv_usec-starttime->tv_usec);
  return usec;

}

bool IsOkToProcess (void)
{
  struct timeval now;
  gettimeofday (&now, 0);
  double elapsedTime = timevaldiff (&startClock, &now);
  if (elapsedTime/1e6 >= 0.10)
      return true;
  else
      return false;
}

void setStartClock (void)
{
    gettimeofday (&startClock, 0);
}

void
AnimSimulatorImpl::Run (void)
{
  if (!mainInfo->GetSimulation()->IsConfigureSimulationComplete())
  {

      QString t = "Simulation::Run called before ConfigureSimulation Completed\n \
                   Please remove any Simulation::Run statements\n \
                   from Simulation::ConfigureSimulation";
      std::cout << t.toAscii().data() <<std::endl;

      exit(-1);
  }
  m_stop = false;
  setStartClock ();
  while (!m_events->IsEmpty () && !m_stop)
    {
      mainInfo->lockSimulation();
      if (!mainInfo->IsSimulationPaused() && !mainInfo->IsSimulationTerminated())
      {
         ProcessOneEvent ();
         mainInfo->unlockSimulation();
         mainInfo->getMainWindow()->SetLCD();
         if (IsOkToProcess())
         {
           QApplication::processEvents();
           setStartClock ();
         }


      }
      else if (mainInfo->IsSimulationTerminated())
      {
        mainInfo->unlockSimulation();
        return;
      }
      else
      {
          mainInfo->unlockSimulation();
          QApplication::processEvents();
      }

    }

  // If the simulator stopped naturally by lack of events, make a
  // consistency test to check that we didn't lose any events along the way.
  NS_ASSERT (!m_events->IsEmpty () || m_unscheduledEvents == 0);
}

void
AnimSimulatorImpl::RunOneEvent (void)
{
  ProcessOneEvent ();
}

void
AnimSimulatorImpl::Stop (void)
{
  m_stop = true;
}

void
AnimSimulatorImpl::Stop (Time const &time)
{
  Simulator::Schedule (time, &Simulator::Stop);
}

//
// Schedule an event for a _relative_ time in the future.
//
EventId
AnimSimulatorImpl::Schedule (Time const &time, EventImpl *event)
{
  Time tAbsolute = time + TimeStep (m_currentTs);

  NS_ASSERT (tAbsolute.IsPositive ());
  NS_ASSERT (tAbsolute >= TimeStep (m_currentTs));
  Scheduler::Event ev;
  ev.impl = event;
  ev.key.m_ts = (uint64_t) tAbsolute.GetTimeStep ();
  ev.key.m_context = GetContext ();
  ev.key.m_uid = m_uid;
  m_uid++;
  m_unscheduledEvents++;
  m_events->Insert (ev);
  return EventId (event, ev.key.m_ts, ev.key.m_context, ev.key.m_uid);
}

void
AnimSimulatorImpl::ScheduleWithContext (uint32_t context, Time const &time, EventImpl *event)
{
  NS_LOG_FUNCTION (this << context << time.GetTimeStep () << m_currentTs << event);

  Scheduler::Event ev;
  ev.impl = event;
  ev.key.m_ts = m_currentTs + time.GetTimeStep ();
  ev.key.m_context = context;
  ev.key.m_uid = m_uid;
  m_uid++;
  m_unscheduledEvents++;
  m_events->Insert (ev);
}

EventId
AnimSimulatorImpl::ScheduleNow (EventImpl *event)
{
  Scheduler::Event ev;
  ev.impl = event;
  ev.key.m_ts = m_currentTs;
  ev.key.m_context = GetContext ();
  ev.key.m_uid = m_uid;
  m_uid++;
  m_unscheduledEvents++;
  m_events->Insert (ev);
  return EventId (event, ev.key.m_ts, ev.key.m_context, ev.key.m_uid);
}

EventId
AnimSimulatorImpl::ScheduleDestroy (EventImpl *event)
{
  EventId id (Ptr<EventImpl> (event, false), m_currentTs, 0xffffffff, 2);
  m_destroyEvents.push_back (id);
  m_uid++;
  return id;
}

Time
AnimSimulatorImpl::Now (void) const
{
  return TimeStep (m_currentTs);
}

Time
AnimSimulatorImpl::GetDelayLeft (const EventId &id) const
{
  if (IsExpired (id))
    {
      return TimeStep (0);
    }
  else
    {
      return TimeStep (id.GetTs () - m_currentTs);
    }
}

void
AnimSimulatorImpl::Remove (const EventId &id)
{
  if (id.GetUid () == 2)
    {
      // destroy events.
      for (DestroyEvents::iterator i = m_destroyEvents.begin (); i != m_destroyEvents.end (); i++)
        {
          if (*i == id)
            {
              m_destroyEvents.erase (i);
              break;
            }
        }
      return;
    }
  if (IsExpired (id))
    {
      return;
    }
  Scheduler::Event event;
  event.impl = id.PeekEventImpl ();
  event.key.m_ts = id.GetTs ();
  event.key.m_context = id.GetContext ();
  event.key.m_uid = id.GetUid ();
  m_events->Remove (event);
  event.impl->Cancel ();
  // whenever we remove an event from the event list, we have to unref it.
  event.impl->Unref ();

  m_unscheduledEvents--;
}

void
AnimSimulatorImpl::Cancel (const EventId &id)
{
  if (!IsExpired (id))
    {
      id.PeekEventImpl ()->Cancel ();
    }
}

bool
AnimSimulatorImpl::IsExpired (const EventId &ev) const
{
  if (ev.GetUid () == 2)
    {
      if (ev.PeekEventImpl () == 0 ||
          ev.PeekEventImpl ()->IsCancelled ())
        {
          return true;
        }
      // destroy events.
      for (DestroyEvents::const_iterator i = m_destroyEvents.begin (); i != m_destroyEvents.end (); i++)
        {
          if (*i == ev)
            {
              return false;
            }
        }
      return true;
    }
  if (ev.PeekEventImpl () == 0 ||
      ev.GetTs () < m_currentTs ||
      (ev.GetTs () == m_currentTs &&
       ev.GetUid () <= m_currentUid) ||
      ev.PeekEventImpl ()->IsCancelled ())
    {
      return true;
    }
  else
    {
      return false;
    }
}

Time
AnimSimulatorImpl::GetMaximumSimulationTime (void) const
{
  // XXX: I am fairly certain other compilers use other non-standard
  // post-fixes to indicate 64 bit constants.
  return TimeStep (0x7fffffffffffffffLL);
}

uint32_t
AnimSimulatorImpl::GetContext (void) const
{
  return m_currentContext;
}

} // namespace ns3

#endif // WITH_NS3
