/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: John Abraham <john.abraham@gatech.edu>
 */

#include  "AnimOnlineParser.h"
#include "Animator/ThreadInfo.h"
#include "MainInfo/MainInfo.h"
#include <iostream>
#include <QImage>
#include "XML/xmltags.h"

using namespace std;


namespace netanim {

// Anim Parser methods
AnimOnlineParser::AnimOnlineParser( unsigned tid , MainInfo * mi)
    : lpId(0), model(0), threadIndex(tid), first(true), mainInfo(mi)
{
}

string AnimOnlineParser::GetNamedAttribute(AttributeList& attributes, const string& which)
{
  string r;
  for (unsigned i = 0; i < attributes.getLength(); ++i)
    {
      const XMLCh* name = attributes.getName(i);
      char* c = XMLString::transcode(name);
      if (string(c) == which)
        { // Found it
          const XMLCh* val = attributes.getValue(i);
          char* v = XMLString::transcode(val);
          r = string(v);
          XMLString::release(&v);
          XMLString::release(&c);
          return r;
        }
      XMLString::release(&c);
    }
  return r;
}

// Element handlers
void AnimOnlineParser::ProcessAnim(AttributeList& attrs)
{
  string lp = GetNamedAttribute(attrs, "lp");
  if (lp.empty())
    {
      cout << "Missing lp attribute on <anim>" << endl;
    }
  lpId = atol(lp.c_str());
}

void AnimOnlineParser::ProcessTopology(AttributeList& attrs)
{
  string x0 = GetNamedAttribute(attrs, minX);
  string y0 = GetNamedAttribute(attrs, minY);
  string x1 = GetNamedAttribute(attrs, maxX);
  string y1 = GetNamedAttribute(attrs, maxY);
  // Add to NetModel
  model->defaultMinMax =
    QRectF(QPointF(atof(x0.c_str()), atof(y0.c_str())),
           QPointF(atof(x1.c_str()), atof(y1.c_str())));
}

void AnimOnlineParser::ProcessNode(AttributeList& attrs)
{
  string lp0 = GetNamedAttribute(attrs, lp);
  string id0 = GetNamedAttribute(attrs, id);
  string locx0 = GetNamedAttribute(attrs, locX);
  string locy0 = GetNamedAttribute(attrs, locY);
  string shape0 = GetNamedAttribute(attrs, shape);
  string image0 = GetNamedAttribute(attrs, image);
  string imageScale0 = GetNamedAttribute(attrs, imageScale);

  Node   n(atol(lp0.c_str()), atol(id0.c_str()),
           atof(locx0.c_str()), atof(locy0.c_str()), 0);
  if (!shape0.empty())
    {
      if (shape0 == "circle") n.shape = Node::CIRCLE;
      else if (shape0 == "square") n.shape = Node::SQUARE;
      else cout << "Unknown node shape " << shape0 << endl;
    }
  n.imageName = image0;
  if (!n.imageName.empty())
    {
      if (!imageScale0.empty()) n.imageScale = atof(imageScale0.c_str());
      n.image = new QImage;
      n.image->load(n.imageName.c_str());
    }
  n.color = GetNamedAttribute(attrs, color);
  model->AddNode(n);
}

void AnimOnlineParser::ProcessLink(AttributeList& attrs)
{
  string fromlp0 = GetNamedAttribute(attrs, fromLp);
  string fromid0 = GetNamedAttribute(attrs, fromId);
  string tolp0 = GetNamedAttribute(attrs, toLp);
  string toid0 = GetNamedAttribute(attrs, toId);
  WiredLink wl(atol(fromlp0.c_str()), atol(fromid0.c_str()),
               atol(tolp0.c_str()), atol(toid0.c_str()));
  model->AddLink(wl);
}

void AnimOnlineParser::ProcessMove(AttributeList& attrs)
{
  string stime = GetNamedAttribute(attrs, startTime);
  string slx   = GetNamedAttribute(attrs, startLocX);
  string sly   = GetNamedAttribute(attrs, startLocY);
  string etime = GetNamedAttribute(attrs, endTime);
  string elx   = GetNamedAttribute(attrs, endLocX);
  string ely   = GetNamedAttribute(attrs, endLocY);
  string lp0    = GetNamedAttribute(attrs, lp);
  string id0    = GetNamedAttribute(attrs, id);
  Mobility m(Point3(atof(slx.c_str()), atof(sly.c_str()), 0),
             atof(stime.c_str()),
             Point3(atof(elx.c_str()), atof(ely.c_str()), 0),
             atof(etime.c_str()));
  model->AddMobility(atol(lp0.c_str()), atol(id0.c_str()), m);
}

void AnimOnlineParser::ProcessQueue(AttributeList& attrs)
{ // code later
    (void) attrs;
}

void AnimOnlineParser::ProcessPacket(AttributeList& attrs, bool w)
{
    string fbtx0 = GetNamedAttribute(attrs, fbTx);
    string lbtx0 = GetNamedAttribute(attrs, lbTx);
    string fromlp0 = GetNamedAttribute(attrs, fromLp);
    string fromid0 = GetNamedAttribute(attrs, fromId);

    pkt.color = GetNamedAttribute(attrs, color);
    pkt.fbTx = atof(fbtx0.c_str());
    pkt.lbTx = atof(lbtx0.c_str());
    pkt.wired = w;
    pkt.txLp = atol(fromlp0.c_str());
    pkt.txId = atol(fromid0.c_str());
    pkt.valid = true;
    pkt.color  = GetNamedAttribute(attrs, color);
    // Set hightest XML time
    ThreadInfo& myInfo = mainInfo->threadInfo[threadIndex];
    myInfo.lastXMLTime = max(myInfo.lastXMLTime, pkt.lbTx);
    myInfo.advancedTo = myInfo.lastXMLTime;
    // We might be advancing too far. check advanceTo value and wait
    // for animator to chatch up if need be.

}

void AnimOnlineParser::ProcessRx(AttributeList& attrs)
{
  string tolp0 = GetNamedAttribute(attrs, toLp);
  string toid0 = GetNamedAttribute(attrs, toId);
  string fbrx0 = GetNamedAttribute(attrs, fbRx);
  string lbrx0 = GetNamedAttribute(attrs, lbRx);
  pthread_mutex_lock(&mainInfo->coutMutex);
  Time_t fbRx = atof(fbrx0.c_str());
  Time_t lbRx = atof(lbrx0.c_str());
  Time_t rxLp = atol(tolp0.c_str());
  Time_t rxId = atol(toid0.c_str());
  pthread_mutex_unlock(&mainInfo->coutMutex);
  // Set hightest XML time
  ThreadInfo& myInfo = mainInfo->threadInfo[threadIndex];
  myInfo.lastXMLTime = max(myInfo.lastXMLTime, lbRx);
  myInfo.advancedTo = myInfo.lastXMLTime;
  pkt.AddRx(fbRx, lbRx, rxLp, rxId, GetNamedAttribute(attrs, color));
  mainInfo->completedThisCycle++;
}
} //namespace netanim

