/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: George F. Riley<riley@ece.gatech.edu>
 * Modified by: John Abraham <john.abraham@gatech.edu>
 */
#include "animdochandlers.h"
#include "Animator/ThreadInfo.h"
#include "MainInfo/MainInfo.h"
#include "XML/xmltags.h"
#include <iostream>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/util/BinFileInputStream.hpp>
#include <xercesc/framework/StdInInputSource.hpp>
#include <xercesc/framework/LocalFileInputSource.hpp>
#include <xercesc/parsers/SAXParser.hpp>
#include <xercesc/sax/HandlerBase.hpp>
#include <xercesc/sax/AttributeList.hpp>
#include <xercesc/sax/SAXParseException.hpp>
#include <xercesc/sax/SAXException.hpp>


using namespace std;
namespace netanim {

AnimDocHandlers::AnimDocHandlers(MainInfo *mi)
    : parser(nil), mainInfo (mi)
{ // Nothing needed
}

AnimDocHandlers::~AnimDocHandlers()
{ // Nothing needed
}

void AnimDocHandlers::endDocument()
{
  //cout << "endDocument " << endl;
}

void AnimDocHandlers::endElement(
                                const XMLCh* const name)
{
  char* cp = XMLString::transcode(name);
  string element(cp);
  // The only end's we care about are packet, wpacket, and anim
  if (element == packet || element == wpacket)
    {
      parser->model->AddPacket(parser->pkt);
      parser->pkt = Packet(); // clear for next pass
    }
  // think about what to do for /anim tags.
  // I think we just need to let main know we are here and continue
  // to claim we advanced.
  if (element == anim)
    {
      ThreadInfo& myInfo = mainInfo->threadInfo[parser->threadIndex];
      myInfo.done = true;
      myInfo.maxTime = myInfo.lastXMLTime;
      pthread_mutex_lock(&mainInfo->coutMutex);
      cout << "Thread " << parser->threadIndex << " done" << endl;
      pthread_mutex_unlock(&mainInfo->coutMutex);
      while(!myInfo.exit)
        {
          SignalAdvanced(parser->threadIndex);
          WaitForAdvance(parser->threadIndex);
        }
    }
  XMLString::release(&cp);
}

void AnimDocHandlers::startDocument()
{
  //cout << "startDocument" << endl;
}


void AnimDocHandlers::startElement(const   XMLCh* const    name,
                  AttributeList&  attrs)
{
  char* cp = XMLString::transcode(name);
  string element(cp);
  //cout << "Start of element " << element << endl;
  //PrintAttributes(attrs);
  // Find the type of element being processed and create the necessary
  // objects
  if (element == anim)
    { // process the lp number here..code later
      parser->ProcessAnim(attrs);
    }
  else if (element == information)
    {
    }
  else if (element == topology)
    {
      parser->ProcessTopology(attrs);
    }
  else if (element == node)
    {
      parser->ProcessNode(attrs);
    }
  else if (element == lnk)
    {
      parser->ProcessLink(attrs);
    }
  else if (element == move)
    {
      parser->ProcessMove(attrs);
    }
  else if (element == queues)
    {
      parser->ProcessQueue(attrs);
    }
  else if (element == packet)
    {
      parser->ProcessPacket(attrs, true); // wired
    }
  else if (element == wpacket)
    {
      parser->ProcessPacket(attrs, false); // wireless
    }
  else if (element == rx)
    {
      parser->ProcessRx(attrs);
    }
  else
    {
      cout << "Unknown element " << element << endl;
    }
  XMLString::release(&cp);
}

void AnimDocHandlers::characters(
                                 const XMLCh* const chars,
                                 const XMLSize_t)
{
  char* cp = XMLString::transcode(chars);
  string cs(cp);
  cs = noEOL(cs);
  // Call the processor at end of processor stack
  //charProcessors.back()(cs);
  //if (!cs.empty())
  //  cout << "characters are " << cs << endl;
  XMLString::release(&cp);
}

void AnimDocHandlers::ignorableWhitespace(const XMLCh* const,
                                          const XMLSize_t)
{
  //cout << "ignorable whitespace, lth " << length << endl;
}

void AnimDocHandlers::resetDocument()
{
};

void AnimDocHandlers::error(const SAXParseException& e)
{
  char* msg = XMLString::transcode(e.getMessage());
  char* sysid = XMLString::transcode(e.getSystemId());
    XERCES_STD_QUALIFIER cerr << "\nError at (file " << sysid
                 << ", line " << e.getLineNumber()
                 << ", char " << e.getColumnNumber()
         << "): " << msg << XERCES_STD_QUALIFIER endl;
  XMLString::release(&sysid);
  XMLString::release(&msg);
}

void AnimDocHandlers::fatalError(const SAXParseException& e)
{
  char* msg = XMLString::transcode(e.getMessage());
  char* sysid = XMLString::transcode(e.getSystemId());
  XERCES_STD_QUALIFIER cerr << "\nFatal Error at (file " << sysid
                            << ", line " << e.getLineNumber()
                            << ", char " << e.getColumnNumber()
                            << "): " << msg << XERCES_STD_QUALIFIER endl;
  XMLString::release(&sysid);
  XMLString::release(&msg);
}


void AnimDocHandlers::warning(const SAXParseException& e)
{
  char* msg = XMLString::transcode(e.getMessage());
  char* sysid = XMLString::transcode(e.getSystemId());
  XERCES_STD_QUALIFIER cerr << "\nWarning at (file " << sysid
                            << ", line " << e.getLineNumber()
                            << ", char     " << e.getColumnNumber()
                            << "): " << msg << XERCES_STD_QUALIFIER endl;
  XMLString::release(&sysid);
  XMLString::release(&msg);
}

void AnimDocHandlers::resetErrors()
{

}
} //namespace netanim
